#uses "std"
#uses "CtrlXml"
#uses "dpDyn"
#uses "CtrlHTTP"
#uses "CtrlPv2Admin"
#uses "classes/auth/OaAuthServerside"
#uses "classes/auth/OaAuthFactory"
#uses "classes/wssServer/WebUiHttpEndpoints"

/* Base Implementation for the HTTP server features needed for
   the UserInterface running on mobile clients and as Desktop-UI
*/

class HttpServer
{
  public HttpServer()
  {
    string user;

    // We only need the OS user on unix
    if ( _UNIX )
    {
      mapping userInfo = getOSUser();
      user = userInfo.value("Name", "");
    }

    if ( _WIN32 || (user == "root") )
    {
      httpPort  = paCfgReadValueDflt(CONFIG_FILE, "webClient_"+myManNum(), "httpPort",   80);
      httpsPort = paCfgReadValueDflt(CONFIG_FILE, "webClient_"+myManNum(), "httpsPort", 443);
    }
    else
    {
      // on unix, ports < 1024 are not available for non-root users
      // therefore open alternative ports
      httpPort  = paCfgReadValueDflt(CONFIG_FILE, "webClient_"+myManNum(), "httpPort",  8080);
      httpsPort = paCfgReadValueDflt(CONFIG_FILE, "webClient_"+myManNum(), "httpsPort", 8079);
    }

    // get server authentication setting
    int httpAuthCfg;
    bool httpAuthUlc = false;

    if ( paCfgReadValue(CONFIG_FILE, "webClient_"+myManNum(), "httpAuth", httpAuthCfg) == 0 )
    {
      if(httpAuthCfg == 1)
      {
        httpAuth = true;
        httpAuthUlc = true;
      }
      else if(httpAuthCfg == 2)  //mma: BSA extension, httpAuth = 2 disables SSO in case of "OaAuthMethodAD"
      {
        httpAuth = "NOSSO";
      }
    }

    OaAuthServerside auth;
    authServerSide = auth;
    isServerSideLoginEnabled = authServerSide.isServerSideAuthEnabled();

    ulcNeedAuth = httpAuthUlc || isServerSideLoginEnabled;
    DebugFTN("HTTP", "ulcNeedAuth", ulcNeedAuth);

    lowestAutoManNumMobileUI = paCfgReadValueDflt(CONFIG_FILE, "webClient_"+myManNum(), "LowestAutoManNumMobileUI", 200);

    switch ( paCfgReadValueDflt(CONFIG_FILE, "webClient_"+myManNum(), "clientProjExt", 0) )
    {
      case 1: // Hostname
      {
        projectExtension = "_" + eventHost().at(0);
        break;
      }
      case 2: // both Redu Hostnames
      {
        if ( isRedundant() )
          projectExtension = "_" + eventHost().at(0) + "_" + eventHost().at(1);
        break;
      }
    }
  }

  public static void setHttpPort(int port) { httpPort = port; }
  public static int getHttpPort() { return httpPort; }

  public static void setHttpsPort(int port) { httpsPort = port; }
  public static int getHttpsPort() { return httpsPort; }

  public static void setHttpAuth(mixed auth) { httpAuth = auth; }
  public static mixed getHttpAuth() { return httpAuth; }

  /* start the http server and open the listener ports
     @return 0 when the server could start and open all ports, otherwise < 0
  */
  public int start()
  {
    if ( isServerSideLoginEnabled )
    {
      // when a ui manager goes online or offline we need to handle this
      dpConnect("cbHandleUiConnections", FALSE, "_Connections.Ui.ManNums", "_Connections_2.Ui.ManNums");
      // connect to SessionTokenTemp to handle tokens coming from more than one
      dpConnect("cbHandleSessionTokens", FALSE, "_System.Auth.SessionTokenInterface");

      // Server Side Login requires the 'Basic' authentication method
      httpAuth = "Basic";

      if ( httpsPort == 0 )
      {
        throwError(makeError("", PRIO_SEVERE, ERR_PARAM, (int)ErrCode::UNEXPECTEDSTATE, "Server Side Authentication requires HTTPS"));
        return -1;
      }
    }
    else if ( httpAuth == UNDEFD_AUTH )
    {
      // SSO requires 'Negotiate' authentication method, this is only selected
      // when there is no httpAuth config entry
      OaAuthFactory authFactory;
      anytype auth = authFactory.getAuth();
      if ( auth.getBaseType() == "OaAuthMethodAD" )
      {
        // OS authentication is activated - this doesn't mean that SSO is activated as well,
        // but further checks are based on the user that tries to log in.
        // The only thing that can be checked is whether the SSO bit is set for this server
        // for at least one group.

        const int SSO_BIT = 31; // bit 32 is on index 31
        string hostname = getHostname();

        // get WS permissions
        dyn_string displayNames;
        dyn_bit32 permissions;
        dpGet("_WsPermission.Permission",  permissions, "_WsPermission.DisplayName", displayNames);

        if (dynlen(permissions) < dynlen(displayNames))
          permissions[dynlen(displayNames)] = 0;

        bool ssoBitSet = false;
        for (int i = 1; i <= dynlen(displayNames); i++)
        {
          if (stricmp(hostname, displayNames[i]) == 0)
          {
            if (getBit(permissions[i], SSO_BIT))
            {
              // there's at least one group for which the SSO_BIT is set for this server,
              // so SSO is potentially possible - use "Negotiate" method so that SSO does
              // work if actually enabled
              ssoBitSet = true;
              break;
            }
          }
        }

        if (ssoBitSet || getKerberosSecurity() > 0)
        {
          //httpAuth = "NTLM";
		        httpAuth = "Negotiate";
          DebugFTN("HTTP", "HTTP authentication method set to Negotiate");
        }
        else
        {
          DebugFTN("HTTP", "OS user authentication activated, but neither the SSO bit is set for this host nor Kerberos is active");
        }
      }
      else
        httpAuth = false;
    }
    else if(httpAuth == "NOSSO") //mma: BSA extension, check also if httpAuth was set to "NOSSO" (due to config entry httpAuth = 2) and set it to false
    {
      httpAuth = false;
    }

    int rc = httpServer(httpAuth, httpPort, httpsPort);  // start http Server

    if ( rc != 0 )
    {
      throwError(makeError("", PRIO_WARNING, ERR_CONTROL, (int)ErrCode::UNEXPECTEDSTATE, "httpServer could not be started"));
      return rc;
    }

    mapping open = makeMapping("authType", "");
    httpSetPermission("/data/html/*", open);
    httpSetPermission("/data/ulc/start.html",   open);
    httpSetPermission("/pictures/*",            open);
    httpSetPermission("/favicon.ico",           open);
    httpSetPermission("/authInfo", open);
    httpConnect("authInfo", "/authInfo");

    // open the download page
    httpSetPermission("/",        open);
    httpSetPermission("/download",        open);
    httpSetPermission("/data/clsetup*",        open);
    httpSetPermission("/images/*",            open);

    httpConnect("handleUlcLogin", "/data/html/login/index.html");
    httpConnect("handleUlcStart", "/data/ulc/ulc.html");  //mma: BSA extension, handles the additional querry parameters (UserGrps and Workstation) in case of BLE login

    int maxLogSize, maxContentSize;

    // check if the maxLogFileSize was set in the config file
    paCfgReadValue(CONFIG_FILE, "general", "maxLogFileSize", maxLogSize);

    // if no value was defined use the default of 10 MB
    if ( maxLogSize == 0 )
      maxLogSize = 10;

    // calculate the max content size in bytes
    maxContentSize = maxLogSize * 1024 * 1024;

    // set the max content size
    httpSetMaxContentLength(maxContentSize);

    // get information if indexPage is set in Config
    string indexPage;
    if ( paCfgReadValue(CONFIG_FILE, "httpServer", "indexPage", indexPage) == -1 )
      httpConnect("redirectToDownload", "/");

    httpConnect("getIndex", "/download");
    httpConnect("workInfo", "/_info", "text/plain");

    // connect endpoints for Dashboard/WebUI
    WebUiHttpEndpoints::connectEndpoints(httpsPort);

    //if serverSideAuth is enabled - set necessary permissions for _info URL
    if (isServerSideLoginEnabled)
    {
      bool bAllowUnknownUsers  = TRUE;
      bool bCheckPassword      = TRUE;
      bool bAllowDisabledUsers = FALSE;
      authServerSide.getHttpPermissions(bAllowUnknownUsers,bCheckPassword,bAllowDisabledUsers);
      httpSetPermission("/_info", makeMapping("allowUsers", "*",
                                              "allowUnknownUsers", bAllowUnknownUsers,
                                              "checkPassword", bCheckPassword,
                                              "allowDisabledUsers", bAllowDisabledUsers));

      // Allow the ULC UX
      mapping permission = makeMapping("allowUnknownUsers", TRUE, "allowDisabledUsers", TRUE);




      httpSetPermission("/UI_LoadBalance",            permission);
      httpSetPermission("/UI_WebSocket",              permission);
    }

    // for security reasons, the config directory can not be accessed directly
    httpConnect("getConfig", "/config/config", "text/plain");
    httpConnect("getStyleSheet", "/config/stylesheet.css", "text/plain");
    httpConnect("getTouchStyleSheet", "/config/touchscreen.css", "text/plain");
    httpConnect("getHostCert", "/config/host-cert.pem", "text/plain");
    httpConnect("getHostKey", "/config/host-key.pem", "text/plain");
    httpConnect("getRootCert", "/config/root-cert.pem", "text/plain");
    // IM 119158 (cstoeg): get also powerconfig file
    httpConnect("getPowerConfig", "/config/powerconfig", "text/plain");
    httpConnect("logFileUpload", "/logFileUpload", "text/plain");

    dpQueryConnectSingle("cbConnectUI", TRUE, "connect", "SELECT '_online.._value' FROM '_Ui_*.DisplayName' WHERE _DPT= \"_Ui\" AND '_online.._value' != \"\" ");
    dpQueryConnectSingle("cbDisconnectUI", FALSE, "disconnect", "SELECT '_online.._value' FROM '_Ui_*.DisplayName' WHERE _DPT= \"_Ui\" AND '_online.._value' == \"\" ");
  }

  //--------------------------------------------------------------------------------
  // protected methods

  /// return answer in a httpConnect() callback when requested resource was not found
  protected static dyn_string notFoundAnswer()
  {
    return makeDynString(NOT_FOUND, "Status: 404 Not Found");
  }

  //--------------------------------------------------------------------------------
  // private methods

  static dyn_string authInfo(dyn_string names, dyn_string values, string user)
  {
    return jsonEncode(makeMapping("needAuth", ulcNeedAuth, "user",  user, "uid", getUserId(user)));
  }

  //--------------------------------------------------------------------------------
  // type ... rpm|tgz

  static string getFileVersion(string name, string type, int &maxServicePack, int &maxPatchNum,
                               string subdir = "", string arch = "*")
  {
    // get the highest servicePack-patchNum combination
    maxServicePack = 0;
    maxPatchNum = 0;

    string fileName;

    // get the filename of the rpm file, which includes the patch number
    // parse the patch number from that filename
    string dir = PVSS_PATH + "/data/clsetup/" + subdir;  // the files must be in pvss_path !
    dyn_string files = getFileNames(dir, "WinCC_OA_" + VERSION + "-" + name + "-*." + arch + "." + type);

    if ( dynlen(files) == 0 )
      return "";

    for (int i = 1; i <= dynlen(files); i++)
    {
      int servicePack;
      int patchNumber;
      sscanf(files[i], "WinCC_OA_" + VERSION + "-" + name + "-%d-%d.", servicePack, patchNumber);

      if ( (i == 1) || (servicePack > maxServicePack) )
      {
        maxServicePack = servicePack;
        maxPatchNum = patchNumber;
        fileName = files[i];
      }
      else if ( (servicePack == maxServicePack) && (patchNumber > maxPatchNum) )
      {
        maxPatchNum = patchNumber;
        fileName = files[i];
      }
    }

    return fileName;
  }

  //--------------------------------------------------------------------------------

  static string getCodeMeterRpm()
  {
    const string dir = PVSS_PATH + "/data/clsetup/";
    dyn_string files = getFileNames(dir, "CodeMeter-*.rpm");

    if ( dynlen(files) == 0 )
      return "";
    else if ( dynlen(files) == 1)
      return files[1];

    // can't make any assumptions about the naming scheme of CodeMeter installers,
    // so just return the most recent one
    string name;
    time newestTs;
    for (int i = 1; i <= dynlen(files); i++)
    {
      time ts = getFileModificationTime(dir + files[i]);
      if ( ts > newestTs )
      {
        name = files[i];
        newestTs = ts;
      }
    }

    return name;
  }

  //--------------------------------------------------------------------------------

  static string getCodeMeterDeb()
  {
    const string dir = PVSS_PATH + "/data/clsetup/";
    dyn_string files = getFileNames(dir, "codemeter_*.deb");

    if ( dynlen(files) == 0 )
      return "";
    else if ( dynlen(files) == 1)
      return files[1];

    // can't make any assumptions about the naming scheme of CodeMeter installers,
    // so just return the most recent one
    string name;
    time newestTs;
    for (int i = 1; i <= dynlen(files); i++)
    {
      time ts = getFileModificationTime(dir + files[i]);
      if ( ts > newestTs )
      {
        name = files[i];
        newestTs = ts;
      }
    }

    return name;
}
//--------------------------------------------------------------------------------

  static dyn_string handleUlcLogin(const dyn_string names, const dyn_string values, const string user)
  {
    string filePath = getPath(DATA_REL_PATH, "html/login/index.html");

    if ( filePath != "" )
    {
      mapping lang;
      mapping json;
      string content;

      fileToString(filePath, content);

      dyn_string langStrings = makeDynString("login", "usernameEmptyText","passwordEmptyText","welcomeText","cancel", "invalidCreditentials");
      for (int i = 1; i <= dynlen(langStrings); i++) {
          lang[langStrings[i]] = getCatStr("http", langStrings[i]);
       }
      json["lang"] = lang;

      int thisMajor = getVersionInfo("major"), thisMinor = getVersionInfo("minor"), thisPatch = getVersionInfo("patch");
      json["productVersion"] = thisMajor + "." + thisMinor + " P" + thisPatch;

      strreplace(content, "{DATA}", jsonEncode(json));

      return makeDynString(content, "Status: 200 OK");
    }

    // return not found (should not happen)
    return notFoundAnswer();
  }

  static string handleUlcStart(dyn_string names, dyn_string values, string user, string ip,
                               dyn_string headerNames, dyn_string headerValues)
  {
    DebugTN("xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx");
    DebugTN("Names", names);
    DebugTN("Values", values);
    DebugTN("User", user);
    DebugTN("IP", ip);
    //DebugTN("headerNames", headerNames);
    //DebugTN("headerValues", headerValues);

    string bleUserGroup;
    string bleWorkstation;

    int groupIndex = names.indexOf("UserGrps");  //mma: if the user group info is contained in the url query parameters
    if(groupIndex != -1)
    {
      bleUserGroup = values.at(groupIndex);
    }

    int workstationIndex = names.indexOf("Workstation");  //mma: if the workstation info is contained in the url query parameters
    if(workstationIndex != -1)
    {
      bleWorkstation = values.at(workstationIndex);
    }

    dpSet("_BsaBleLogin.userGroup", bleUserGroup,
          "_BsaBleLogin.workstation", bleWorkstation);  //mma: set the information to a internal dp so it can be grapped afterwards by the UI Manager
    /*
      Note: using this internal datapoint leaves a room for a raise conditions if two ULC UIs connecte at allmost the same time,
        but currently there is no way to forwared the query parameters from the url directly to the UI Manager that will be started
      */

    string s = "";
    string htmlFile = getPath(DATA_REL_PATH, "ulc/ulc.html");

    fileToString(htmlFile, s);
    return s;
  }

  //--------------------------------------------------------------------------------

  static dyn_string redirectToDownload()
  {
    return makeDynString("", "Status: 301 Moved Permanently", "Location: /download");
  }

  //--------------------------------------------------------------------------------

  static dyn_string getIndex(dyn_string names, dyn_string values, string user, string ip,
                             dyn_string headerNames, dyn_string headerValues)
  {
    string filePath = getPath(DATA_REL_PATH, "webclient_index.html");
    if ( filePath == "" )  // does not exist
      return notFoundAnswer();

    string content;
    fileToString(filePath, content);

    // get the highest servicePack-patchNum combination
    int maxServicePack = 0;
    int maxPatchNum = 0;

    // This is a workaround for the fact that desctop UI package for RHEL has changed name
    // in 3.18 and therefore do not fit the name scheme used by getFileVersion(). The only RPM
    // package still using the old name scheme is the ITC package, so use this to get the latest
    // patch number (service pack is ignored anyway
    // if ( getFileVersion("desktop-ui-rhel", "rpm", maxServicePack, maxPatchNum, "linux-rhel-x86_64/") == "" )
    if ( getFileVersion("ui-webruntime", "tgz", maxServicePack, maxPatchNum, "itcv3/") == "" )
      return notFoundAnswer();

    string RPM1="linux-rhel-x86_64/WinCCOA-" + VERSION + "." + (string)maxPatchNum + "-DesktopUI-rhel";
    string codeMeterRpm=getCodeMeterRpm();
    string codeMeterDeb=getCodeMeterDeb();
    string RPM4="linux-debian-x86_64/WinCCOA-" + VERSION + "." + (string)maxPatchNum + "-DesktopUI-debian";

    // the download buttons:
    string download;
    string codemeterDownload;

    // check the OS of the client
    int idx = dynContains(headerNames, "User-Agent");
    if ( idx > 0 )
    {
      string os = strtolower(headerValues[idx]);


      if ( strpos(os, "linux") != -1 )
      {
          download =
              "<button class='button' "
              "onclick='document.location.href=\"/data/clsetup/" + RPM4 + ".x86_64.deb\";'"
              "title='Download Desktop UI for Industrial OS x86-64'>"
              "Industrial OS x86-64"
              "</button>";
          download +=
              "<button class='button' "
              "onclick='document.location.href=\"/data/clsetup/" + RPM1 + ".x86_64.rpm\";'"
              "title='Download Desktop UI for Oracle Linux / Linux RedHat x86-64'>"
              "Oracle Linux / Linux RedHat x86-64"
              "</button>";

          codemeterDownload =
              "<button class='button' "
              "onclick='document.location.href=\"/data/clsetup/" + codeMeterDeb + "\";'"
              "title='Download CodeMeter for Industrial OS x86-64'>"
              "Industrial OS x86-64"
              "</button>";
          codemeterDownload +=
              "<button class='button' "
              "onclick='document.location.href=\"/data/clsetup/" + codeMeterRpm + "\";'"
              "title='Download CodeMeter for Oracle Linux / Linux RedHat x86-64'>"
              "Oracle Linux / Linux RedHat x86-64"
              "</button>";

      }
      else if ( strpos(os, "windows") != -1 )
      {
        if ( patternMatch("*i?86*", os) ||     // browser is 32bit
             (strpos(os, "nt 5.1") != -1) )    // Windows XP is always 32bit
        {
          download =
            "<div style='padding-top:10px; text-align:center;'>"
              "32-Bit Systems are not supported"
            "</div>";
        }
        else if ( (strpos(os, "x64") != -1) || (strpos(os, "win64") != -1) || (strpos(os, "wow64") != -1))    // browser is 64bit
        {
          download =
              "<button class='button' "
              "onclick='document.location.href=\"/data/clsetup/windows-64/WinCC_OA_Desktop_UI_" + VERSION + "-64.exe\";'"
              "title='Download Desktop UI for Windows 64-bit'>"
              "Windows 64-bit"
              "</button>";
        }
        else // CPU unknown; let the user select
        {
          download =
              "<button class='button' "
              "onclick='document.location.href=\"/data/clsetup/windows-64/WinCC_OA_Desktop_UI_" + VERSION + "-64.exe\";'"
              "title='Download Desktop UI for Windows 64-bit'>"
              "Windows 64-bit"
              "</button>";
        }
        strreplace(content, "<h2>CodeMeter Tool / Licensing</h2>", "");
      }
    }

    if ( download == "" )  // no (matching) OS found; let the user choose
    {
          download =
              "<button class='button' "
              "onclick='document.location.href=\"/data/clsetup/windows-64/WinCC_OA_Desktop_UI_" + VERSION + "-64.exe\";'"
              "title='Download Desktop UI for Windows 64-bit'>"
              "Windows 64-bit"
              "</button>";
          download +=
              "<button class='button' "
              "onclick='document.location.href=\"/data/clsetup/" + RPM4 + ".x86_64.deb\";'"
              "title='Download Desktop UI for Industrial OS x86-64'>"
              "Industrial OS x86-64"
              "</button>";
          download +=
              "<button class='button' "
              "onclick='document.location.href=\"/data/clsetup/" + RPM1 + ".x86_64.rpm\";'"
              "title='Download Desktop UI for Oracle Linux / Linux RedHat x86-64'>"
              "Oracle Linux / Linux RedHat x86-64"
              "</button>";

          codemeterDownload =
              "<button class='button' "
              "onclick='document.location.href=\"/data/clsetup/" + codeMeterDeb + "\";'"
              "title='Download CodeMeter for Industrial OS x86-64'>"
              "Industrial OS x86-64"
              "</button>";
          codemeterDownload +=
              "<button class='button' "
              "onclick='document.location.href=\"/data/clsetup/" + codeMeterRpm + "\";'"
              "title='Download CodeMeter for Oracle Linux / Linux RedHat x86-64'>"
              "Oracle Linux / Linux RedHat x86-64"
              "</button>";
    }

    strreplace(content, "<%DOWNLOAD>", download);
    strreplace(content, "<%DOWNLOAD_CODEMETER>", codemeterDownload);

    return makeDynString(content, "Status: 200 OK");
  }

  //--------------------------------------------------------------------------------

  static anytype workInfoClientUpdateCheck(const dyn_string &names, const dyn_string &values,
                                           const string &user, const string &ip,
                                           const dyn_string &headerNames, const dyn_string &headerValues,
                                           const int &connIdx)
  {
    if ( dynlen(names) < 5 ||
         dynContains(names, "major") <= 0 ||
         dynContains(names, "minor") <= 0 ||
         dynContains(names, "patch") <= 0 ||
         dynContains(names, "arch") <= 0 )
      return makeDynString("", "Status: 400 Bad Request");

    //TODO: check if strings are really numerics
    string cliMajor = values[dynContains(names, "major")];
    string cliMinor = values[dynContains(names, "minor")];
    string cliPatch = values[dynContains(names, "patch")];
    string cliArch = values[dynContains(names, "arch")];

    int cliLang = httpGetLanguage(connIdx);

    int thisMajor = getVersionInfo("major"), thisMinor = getVersionInfo("minor"), thisPatch = getVersionInfo("patch");

    DebugFTN("HTTP", "Client", cliMajor, cliMinor, cliPatch, cliArch, cliLang);
    DebugFTN("HTTP", "Server", thisMajor, thisMinor, thisPatch);

    string content = "";
    mapping m = makeMapping("major", thisMajor,
                            "minor", thisMinor,
                            "patch", thisPatch);

    if ((string)thisMajor == cliMajor &&
        (string)thisMinor == cliMinor)
    {
      DebugFTN("HTTP", "Version matches");

      if (thisPatch <= ((int)cliPatch))
      {
        DebugFTN("HTTP", "Patchlevel is OK for connection");

        m["connect"] = "OK";
        content = jsonEncode(m);

        return makeDynString(content, "Status: 200 OK");
      }
      else
      {
        DebugFTN("HTTP", "newer patch available.");
        // return new patch
        if (cliArch == "windows-64")
        {
          m["patchURL"] = "/data/clsetup/windows-64/WinCC_OA_Desktop_UI_" + VERSION + "-64.exe";
        }
        else if (cliArch == "windows")
        {
          m["patchURL"] = "/data/clsetup/windows/WinCC_OA_Desktop_UI_" + VERSION + ".exe";
        }
        else
        {
          int checkServicepack=0, checkPatch=0;

          string fn="";
          string name=cliArch;
          strreplace(name, "linux-", "");
          strreplace(name, "-x86_64", "");
          name="desktop-ui-" + name;
          fn=getFileVersion(name, "rpm", checkServicepack, checkPatch, cliArch + "/");

          if (fn == "" ||
              checkServicepack != 0 ||
              checkPatch != thisPatch)
          {
            DebugFTN("HTTP", "no valid patch package found");
            m["infoText"] = getCatStr("http", "rtUiOutdatedNoNewer", cliLang);
            m["connect"] = "OK";
            m["patchURL"] = "/";
            content = jsonEncode(m);
            return makeDynString(content, "Status: 200 OK");
          }

          m["patchURL"] = "/data/clsetup/" + cliArch + "/" + fn;
        }

        DebugFTN("HTTP", "Found patch for architecture", m["patchURL"]);

        m["infoText"] = getCatStr("http", "rtUiOutdatedNewer", cliLang);
        m["connect"] = "OK";
        content = jsonEncode(m);
        return makeDynString(content, "Status: 200 OK");
      }
    }
    else
    {
      DebugFTN("HTTP", "Version does not match server");
      m["infoText"] = getCatStr("http", "rtUiNoMatch", cliLang);
      m["connect"] = "NOK";
      m["patchURL"] = "/";

      content = jsonEncode(m);
      return makeDynString(content, "Status: 200 OK");

    }

    return makeDynString("", "Status: 400 Bad Request");
  }

  //--------------------------------------------------------------------------------

  static anytype workInfoUiWebRuntimeVersion(const dyn_string &names, const dyn_string &values)
  {
    int thisMajor, thisMinor;
    sscanf(VERSION, "%d.%d", thisMajor, thisMinor);

    string arch = "*";    // architecture
    string type = "rpm";  // package type

    string current = values[1];  // currently installed version on client (or empty)
    int currentMajor = 0, currentMinor = 0;
    int currentServicePack = 0, currentPatchNum = 0;

    int i = strpos(current, "ui-webruntime");
    if ( i >= 0 )
    {
      sscanf(current, "WinCC_OA_%d.%d-ui-webruntime-%d-%d.%s",
             currentMajor, currentMinor,
             currentServicePack, currentPatchNum, arch);
    }
    else
    {
      // check for requested architecture if no rpm is already installed
      int idx = dynContains(names, "arch");
      if ( idx >= 1 )
        arch = values[idx];
    }

    string sub;  // special variation of uiWebRuntime, e.g. for ITC
    int idx = dynContains(names, "sub");
    if ( idx >= 1 )
      sub = values[idx];

    idx = dynContains(names, "type");  // package type
    if ( idx >= 1 )
      type = values[idx];

    // get the highest servicePack-patchNum combination
    int maxServicePack, maxPatchNum;

    string fileName = getFileVersion("ui-webruntime", type, maxServicePack, maxPatchNum, sub, arch);

    if ( fileName == "" )
      return "";

    // create sortable strings, e.g. "Ver_SP_PPP"
    string myVersion, clVersion;
    sprintf(myVersion, "%02d%02d_%02d_%03d", thisMajor, thisMinor, maxServicePack, maxPatchNum);
    sprintf(clVersion, "%02d%02d_%02d_%03d", currentMajor, currentMinor, currentServicePack, currentPatchNum);
    DebugFN("HTTP", "myVersion", myVersion, "clientVersion", clVersion);

    if ( myVersion > clVersion )
      return "/data/clsetup/" + sub + '/' + fileName;
    else
      return "";   // nothing to download
  }

  //--------------------------------------------------------------------------------

  static anytype workInfoUuid(const dyn_string &names, const dyn_string &values,
                              const string &user, const string &ip,
                              const dyn_string &headerNames, const dyn_string &headerValues,
                              int connIdx)
  {
    // get Data - UUID
    int idx = dynContains(names, "uuid");
    string uuid;

    if ( idx >= 1 )
      uuid = values[idx];

    // get Data - Model
    idx = dynContains(names, "model");
    string model;

    if ( idx >= 1 )
      model = values[idx];

    //TFS 15236: ULC UX shall not be handeled at all by device management
    if(model == "ulc")
      return "";

    // get Data - Name
    idx = dynContains(names, "name");
    string sname;

    if ( idx >= 1 )
      sname = values[idx];

    // get Data - Width
    idx = dynContains(names, "w");
    int w;

    if ( idx >= 1 )
      w = (int)values[idx];

    // get Data - Height
    idx = dynContains(names, "h");
    int h;

    if ( idx >= 1 )
      h = (int)values[idx];

    // get Data - DPI
    idx = dynContains(names, "dpi");
    int dpi;

    if ( idx >= 1 )
      dpi = (int)values[idx];

    idx = dynContains(headerNames, "User-Agent");
    string appVersion;

    //App Version
    if ( idx >= 1)
    {
      dyn_string dsTemp;

     dsTemp = strsplit(headerValues[idx]," ");
     appVersion = dsTemp[3];

    }

    int cliLang = httpGetLanguage(connIdx);

    // .... check uuid
    dyn_string dpDeviceNames;
    dyn_string dsUUID, dsArguments, dsDeviceClass;
    dyn_bool dbunlocked, dbAutoLogin;
    dyn_int diManagerNum;
    mapping data;

    //Get Device Data from Datapoint
    dpGet(MOBILE_UI_PRAEFIX_DP + ".DisplayName", dpDeviceNames,
          MOBILE_UI_PRAEFIX_DP + ".UUID", dsUUID,
          MOBILE_UI_PRAEFIX_DP + ".DeviceClass", dsDeviceClass,
          MOBILE_UI_PRAEFIX_DP + ".AutoLogin", dbAutoLogin,
          MOBILE_UI_PRAEFIX_DP + ".Unlocked", dbunlocked,
          MOBILE_UI_PRAEFIX_DP + ".ManagerNumber", diManagerNum);

    int iDynIndex = dynContains(dsUUID, uuid);

    //Case: UUID does not exist
    if (iDynIndex < 1)
    {
      //create datapoint with UUID and model
      dyn_string dsDeviceClasses;
      string sDeviceClass = "";
      dyn_int diHeight, diWidth;
      bool bMobileAutoUnlock, bRuntimeAutoUnlock;
      float fRatioDevice = (float)w/(float)h, fRatioClass_wh, fRatioClass_hw;

      // define device class for mobile Ui
      if ( model != "desktop" )
      {
        if ( dpi > 0 )  // screen diagonal in Inches
        {
          float deviceDiagonal = sqrt((w / dpi) * (w / dpi) + (h / dpi) * (h / dpi));
          sDeviceClass = (deviceDiagonal > MAX_SMARTPHONE_SCREEN) ? "tablet" : "smartphone";
        }
        else
        {
          sDeviceClass = "smartphone";
        }
      }

      //create Mapping
      data["arguments"] = "";
      data["deviceClass"] = sDeviceClass;
      data["deviceLocked"] = TRUE;

      int iManNum = 0;
      if (model != "desktop")
      {
        // Find a new fixed UI manager number if not a Desktop UI has connected
        for (int i = lowestAutoManNumMobileUI; iManNum == 0 && i < 255; i++)
        {
          if (dynContains(diManagerNum, i) == 0)
            iManNum = i;
        }

        if (iManNum == 0)
        {
          throwError(makeError("", PRIO_SEVERE, ERR_PARAM, (int)ErrCode::UNEXPECTEDSTATE,
                               getCatStr("http", "noManNumError", cliLang), sname, model));

          // return the error to the mobile UI
          data["deviceLocked"] = TRUE;
          data["reason"] = "noManNum";

          data["infoText"] = getCatStr("http", "noManNumInfo", cliLang);
          data["titleText"] = getCatStr("http", "noManNumTitle", cliLang);
          return jsonEncode(data);
        }
      }

      dpGet(MOBILE_UI_PRAEFIX_DP_2 + ".AutoValidationEnabled", bMobileAutoUnlock,
            MOBILE_UI_PRAEFIX_DP_2 + ".AutoRuntimeValidationEnabled", bRuntimeAutoUnlock,
            MOBILE_UI_PRAEFIX_DP_2 + ".Name", dsDeviceClasses,
            MOBILE_UI_PRAEFIX_DP_2 + ".Resolution.Width", diHeight,
            MOBILE_UI_PRAEFIX_DP_2 + ".Resolution.Height", diWidth);

      //Case: AutoUnlock is enabled
      if ( (bMobileAutoUnlock && model != "desktop") || (bRuntimeAutoUnlock && model == "desktop") )
      {
        data["deviceLocked"] = FALSE;

        if (model != "desktop")
        {
          data["arguments"] = "-num " + iManNum;

          if (dpExists("_Ui_" + iManNum) == FALSE)
          {
            int iRet;

            iRet = dpCreate("_Ui_" + iManNum, "_Ui");

            if (dynlen(getLastError()) > 0 || iRet)
            {
              throwError(makeError("", PRIO_SEVERE, ERR_SYSTEM, (int)ErrCode::UNEXPECTEDSTATE,
                                   "Could not perform auto unlock, because internal datapoint: _Ui_" + iManNum + " could not be created"));

              data["deviceLocked"] = TRUE;
            }
            else
              dpSet("_Ui_" + iManNum + ".UserName:_archive.._type", DPCONFIG_DEFAULTVALUE,
                    "_Ui_" + iManNum + ".UserName:_archive.._archive", TRUE);
          }
        }
      }

      // define DP elements
      dyn_string dsDpe = makeDynString(MOBILE_UI_PRAEFIX_DP + ".DisplayName",
                                       MOBILE_UI_PRAEFIX_DP + ".UUID",
                                       MOBILE_UI_PRAEFIX_DP + ".ProductModel",
                                       MOBILE_UI_PRAEFIX_DP + ".ManagerNumber",
                                       MOBILE_UI_PRAEFIX_DP + ".DeviceClass",
                                       MOBILE_UI_PRAEFIX_DP + ".Unlocked",
                                       MOBILE_UI_PRAEFIX_DP + ".AutoLogin",
                                       MOBILE_UI_PRAEFIX_DP + ".AppVersion");

      // define values for the DP elements
      dyn_anytype daValues = makeDynAnytype(sname,
                                            uuid,
                                            model,
                                            iManNum,
                                            sDeviceClass, // Deviceclass Vorschlag aufgrund der Hoehe und Breite
                                            !data["deviceLocked"],
                                            FALSE,
                                            appVersion);

      dpDynAppend(dsDpe, daValues);

      dpSet(MOBILE_UI_PRAEFIX_DP_2 + ".NewDevice", TRUE);
    }
    else
    {
      //Case: UUID already exists

      //Update App Version
      dyn_string dsDpe = makeDynString(MOBILE_UI_PRAEFIX_DP + ".AppVersion");
      dyn_anytype daValues = makeDynAnytype(appVersion);
      dpDynIdxSet(iDynIndex, dsDpe, daValues);

      //create Mapping - arguments, deviceclass, locked, rootpanel
      if (model != "desktop")
        data["arguments"] = "-num " + diManagerNum[iDynIndex];

      data["deviceClass"] = dsDeviceClass[iDynIndex];
      data["deviceLocked"] = !dbunlocked[iDynIndex];
    }

    return jsonEncode(data);
  }

  //--------------------------------------------------------------------------------

  static anytype workInfoSessionToken(const dyn_string &names, const dyn_string &values,
                                      const string &user, const string &ip,
                                      const dyn_string &headerNames, const dyn_string &headerValues,
                                      int connIdx)
  {
    // Default return value is an empty session token
    string type = values[1];
    mapping data = makeMapping("sessionToken", "");
    string host = httpGetHeader(connIdx, "Host");

    if ( isServerSideLoginEnabled )
    {
      // Do not allow server side authentication over an unsecure connection
      if (!patternMatch("*:" + httpsPort, host))
      {
        throwError(makeError("", PRIO_SEVERE, ERR_PARAM, (int)ErrCode::UNEXPECTEDSTATE, "Server Side Authentication requires HTTPS"));

        anytype any;
        return any;
      }

      dyn_string error;
      OaAuthServerside auth;

      //if since last login the authentication method has changed the variable needs a new instance of an authServerside object
      if ( auth.getAuthType() != authServerSide.getAuthType() )
      {
        authServerSide = auth;
      }

      data = authServerSide.workInfoSessionToken(user, headerNames, headerValues, type, error);

      if (dynlen(error) > 0)
      {
        return error;
      }
    }

    return jsonEncode(data);
  }

  //--------------------------------------------------------------------------------
  // return some information about the project

  static anytype workInfo(dyn_string names, dyn_string values, string user, string ip,
                          dyn_string headerNames, dyn_string headerValues, int connIdx)
  {
    if ( dynlen(names) == 0 )
      return makeDynString("", "Status: 400 Bad Request");

    if ( names[1] == "projectName" )
    {
      string displayName;
      paCfgReadValue(getPath(CONFIG_REL_PATH) + "config", "general", "displayName", displayName);
      if ( displayName != "" )
        return displayName;
      else
        return PROJ + projectExtension;
    }

    if ( names[1] == "wccoaVersion" )
      return VERSION;

    if ( names[1] == "serverTime" )
      return formatTimeUTC(values[1], getCurrentTime());

    if ( names[1] == "clientUpdateCheck" )
      return workInfoClientUpdateCheck(names, values, user, ip, headerNames, headerValues, connIdx);

    if ( names[1] == "uiWebRuntimeVersion" )
      return workInfoUiWebRuntimeVersion(names, values);

    if ( names[1] == "uuid" )
      return workInfoUuid(names, values, user, ip, headerNames, headerValues, connIdx);

    if ( names[1] == "sessionToken" )
      return workInfoSessionToken(names, values, user, ip, headerNames, headerValues, connIdx);

    // retrieve a list of files /*and their last modified timestamp*/ (in XML format)
    // merged from all proj_paths

    // parameter 1: directory for recursive search or "|" separated list of directories for non recursively search
    // optional parameter 2: project names for searchleves to include; by default all search levels are included
    if ( names[1] == "listFiles" )
    {
      dyn_string files;
      if (!accessAllowed(values[1]))
        return getCatStr("http", "00010");

      DebugFTN(STDDBG_FILEOP, "WEBCLIENT", "names", names, "values", values);

      dyn_string dsDirectories = stringToDynString(values[1]);
      dyn_string dsSearchLevelFilter;

      if (dynlen(values) >= 2) // search only in special levels
        dsSearchLevelFilter = stringToDynString(values[2]);

      // INFO: Pattern match is done here in order to keep this "listFiles" feature generic
      files = getFileNamesLocal(dsDirectories, "*", dsSearchLevelFilter);

      int doc = xmlNewDocument();
      int root = xmlAppendChild(doc, -1, XML_ELEMENT_NODE, "fileList");

      for (int i = 1; i <= dynlen(files); i++)
      {
        if ( baseName(files[i][0]) == '.' )
          continue;  // hidden file, e.g. .colorDB.lock

        int node = xmlAppendChild(doc, root, XML_ELEMENT_NODE, "file");
        xmlSetElementAttribute(doc, node, "name", files[i]);

        /* TODO can be sent (if the file hierarchy-path is found) to speed up init
        xmlSetElementAttribute(doc, node, "lastModified",
            getFileModificationTime(PROJ_PATH + values[1] + "/" + files[i]));
        */
      }

      string strDoc = xmlDocumentToString(doc);
      xmlCloseDocument(doc);  // free memory
      DebugFN("HTTP", strDoc);
      return strDoc;
    }

    return notFoundAnswer();
  }

  //--------------------------------------------------------------------------------
  // add the hosts defined in [general] data or event entry to the hosts list to be
  // used for the automatically generated mxProxy entry for the Desktop or Mobile UI

  static void processDataOrEventEntry(const dyn_string &dsEntry, dyn_string &dsHostsForProxy)
  {
    for (int i = 1; i <= dynlen(dsEntry); i++)
    {
      dyn_string dsReduPairs = strsplit(dsEntry[i], "$");
      for (int j = 1; j <= dynlen(dsReduPairs); j++)
      {
        dyn_string dsReduNet = strsplit(dsReduPairs[j], ",");
        for (int k = 1; k <= dynlen(dsReduNet); k++)
        {
          dyn_string host = strsplit(dsReduNet[k], ":");
          dynAppend(dsHostsForProxy, strtolower(host[1]));
        }
      }
    }
  }

  //--------------------------------------------------------------------------------
  // return the config file (a generated version containing already all config.* files,
  // e.g. config.level, config.<OS>, config.redu

  static dyn_string getConfig(dyn_string names, dyn_string values, string user, string ip,
                              dyn_string headerNames, dyn_string headerValues)
  {
    // Check if there is a special config file for the webclient
    string filePath = getPath(CONFIG_REL_PATH, "config.webclient");

    if ( filePath == "" )  // no, use normal config file
      filePath = getPath(CONFIG_REL_PATH, "config");

    string tmpConfig = tmpnam();
    if ( !copyFile(filePath, tmpConfig) )
    {
      throwError(makeError("", PRIO_WARNING, ERR_SYSTEM, (int)ErrCode::UNEXPECTEDSTATE,
                           "Copying config-file " + filePath + " to temp-config-file " + tmpConfig + " not possible"));
      return makeDynString("", "Status: 500 Internal Server Error");  // Kopiervorgang ist schief gegangen
    }

    dyn_string dsProjPathes;
    paCfgReadValueList(tmpConfig, "general", "proj_path", dsProjPathes);
    for (int i = 1; i <= dynlen(dsProjPathes); i++)
      paCfgDeleteValue(tmpConfig, "general", "proj_path");

    // we need to store every host which is reachable via mxProxy
    // so we can send this information to the client
    dyn_string dsHostsForProxy;

    // we add event to the config to be transferred to client
    dyn_string dsEvents, dsEventHost;
    paCfgReadValueList(tmpConfig, "general", "event", dsEvents);
    paCfgReadValueList(tmpConfig, "general", "eventHost", dsEventHost);

    if (dynlen(dsEventHost) > 0)
    {
      for (int i = 1; i <= dynlen(dsEventHost); i++)
      {
        dyn_string host = strsplit(strtolower(dsEventHost[i]), "$");
        dynAppend(dsHostsForProxy, host);
      }
    }
    else
    {
      if (dynlen(dsEvents) > 0)
      {
        processDataOrEventEntry(dsEvents, dsHostsForProxy);
      }
      else
      {
        //we have only one event manager in this case (single system)
        dyn_string host = eventHost();
        string target = host[1] + ":" + eventPort();
        dynAppend(dsHostsForProxy, strtolower(host[1]));
        paCfgInsertValue(tmpConfig, "general", "event", target);
      }
    }

    // we add data to the config to be transferred to client
    dyn_string dsDatas, dsDataHost;
    paCfgReadValueList(tmpConfig, "general", "data", dsDatas);
    paCfgReadValueList(tmpConfig, "general", "dataHost", dsDataHost);

    if (dynlen(dsDataHost) > 0)
    {
      for (int i = 1; i <= dynlen(dsDataHost); i++)
      {
        dyn_string host = strsplit(strtolower(dsDataHost[i]), "$");
        dynAppend(dsHostsForProxy, host);
      }
    }
    else
    {
      if (dynlen(dsDatas) > 0)
      {
        processDataOrEventEntry(dsDatas, dsHostsForProxy);
      }
      else
      {
        //we have only one data manager in this case (single system)
        dyn_string host = dataHost();
        string target = host[1] + ":" + dataPort();
        dynAppend(dsHostsForProxy, strtolower(host[1]));
        paCfgInsertValue(tmpConfig, "general", "data", target);
      }
    }

    dynUnique(dsHostsForProxy);
    dyn_string dsCurMxProxy;
    paCfgReadValueList(tmpConfig, "general", "mxProxy", dsCurMxProxy);

    if ( dynlen(dsHostsForProxy) > 0 &&
         ( dynlen(dsCurMxProxy) == 0 || dsCurMxProxy[1] != "none" )
       )
    {
      // find which host did the client call, it should also have a running mxProxy
      int hostIndex = 0;
      if ( (hostIndex = dynContains(headerNames, "Host")) > 0 )
      {
        // get the hostname which was used to reach the http(s) endpoint
        dyn_string headerHost = strsplit(strtolower(headerValues[hostIndex]), ":");

        // only if host in the http header is completely unrelated to
        // data and event hosts, add the mxProxy entries
        if (dynContains(dsHostsForProxy, headerHost[1]) == 0)
        {
          int proxyPort = paCfgReadValueDflt(tmpConfig, "proxy", "proxyPort", 5678);

          for (int i = 1; i <= dynlen(dsHostsForProxy); i++)
          {
            paCfgInsertValue(tmpConfig, "general", "mxProxy", dsHostsForProxy[i] + " " + headerHost[1] + ":" + proxyPort + " cert");
          }
        }
      }
    }

    string config;
    fileToString(tmpConfig, config);
    remove(tmpConfig);

    // from WinCC_OA install dir down to project dir
    dyn_string configFiles;
    dynAppend(configFiles, "config.level");

    // add the config.<OS> from the clients OS if possible
    int idx = dynContains(headerNames, "User-Agent");
    if ( idx > 0 )
    {
      string os = strtolower(headerValues[idx]);
      if ( strpos(os, "linux") != -1 )
        dynAppend(configFiles, "config.linux");
      else if ( strpos(os, "windows") != -1 )
        dynAppend(configFiles, "config.nt");
      else if ( strpos(os, "sunos") != -1 )
        dynAppend(configFiles, "config.solaris");
    }

    if ( isRedundant() )
      dynAppend(configFiles, "config.redu");

    for (int i = SEARCH_PATH_LEN; i >= 1; i--)
    {
      for (int j = 1; j <= dynlen(configFiles); j++)
      {
        filePath = getPath(CONFIG_REL_PATH, configFiles[j], 0, i);
        if ( filePath != "" )
        {
          string content;
          fileToString(filePath, content);
          config += "\n## " + filePath + " ##\n";
          config += content;
        }
      }
    }

    return makeDynString(config, "Status: 200 OK");
  }

  //--------------------------------------------------------------------------------
  // read all stylesheet files from all proj-paths and return the merged contents

  static string getMergedStylesheets(const string &fileName)
  {
    string mergedContents;

    for (int i = SEARCH_PATH_LEN; i >= 1; i--)
    {
      string filePath = getPath(CONFIG_REL_PATH, fileName, 0, i);
      if ( filePath != "" )
      {
        string contents;
        fileToString(filePath, contents);
        mergedContents += "\n/* " + filePath + " */\n";
        mergedContents += contents;
      }
    }

    return mergedContents;
  }

  //--------------------------------------------------------------------------------
  // return the config/stylesheet.css file

  static dyn_string getStyleSheet()
  {
    string contents = getMergedStylesheets("stylesheet.css");

    if ( contents == "" )
      return notFoundAnswer();
    else
      return makeDynString(contents, "Status: 200 OK");
  }

  //--------------------------------------------------------------------------------
  // IM 119158: return the config/powerconfig

  static dyn_string getPowerConfig()
  {
    string contents;
    for (int i = 1; i <= SEARCH_PATH_LEN; i++) //search in Project first
    {
      string filePath = getPath(CONFIG_REL_PATH, "powerconfig", 0, i);
      if ( filePath != "" )
      {
        contents = "";
        bool ret;
        ret = fileToString(filePath, contents);
        if ( (ret != 0) && (contents != "") ) //jump out if a powerconfig file is found
          break;
      }
    }

    if ( contents == "" )
      return notFoundAnswer();
    else
      return makeDynString(contents, "Status: 200 OK");
  }

  //--------------------------------------------------------------------------------
  // upload the sent logfile from mobile app

  static void logFileUpload(blob content, string user, string ip, dyn_string headernames, dyn_string headervalues, int connIdx)
  {
    int index;
    index = dynContains(headernames, "X-UUID");
    if ( index > 0 )
    {
      // found UUID in header
      string uuid = headervalues[index];

      dyn_string dsUUID;
      dyn_bool dbUnlocked;
      dyn_int diManagerNumbers;
      dyn_string dsDisplayNames;
      dpGet(MOBILE_UI_PRAEFIX_DP + ".UUID", dsUUID,
            MOBILE_UI_PRAEFIX_DP + ".Unlocked", dbUnlocked,
            MOBILE_UI_PRAEFIX_DP + ".ManagerNumber", diManagerNumbers,
            MOBILE_UI_PRAEFIX_DP + ".DisplayName", dsDisplayNames);

      index = dynContains(dsUUID, uuid);
      if ( index > 0 )
      {
        // UUID exists
        if ( dbUnlocked[index] )
        {
          // OK - save logfiles
          string contentType = httpGetHeader(connIdx, "Content-Type");
          int pos = strpos(contentType, "boundary=");
          if (pos >= 0)
          {
            string boundary = substr(contentType, pos + 9);

            // Hochkomma (") am Beginn und Ende entfernen
            boundary = substr(boundary, 1);
            boundary = substr(boundary, 0, strlen(boundary) - 1);

            mapping result;
            string saveDir = "";
            if ( diManagerNumbers[index] == 0 )  // desktop --> ManNr. 0
              saveDir = PROJ_PATH + LOG_REL_PATH + "desktopUI_logs_ui" + dsDisplayNames[index] + "/";
            else
              saveDir = PROJ_PATH + LOG_REL_PATH + "mobileUI_logs_ui" + diManagerNumbers[index] + "/";
            mkdir(saveDir);
            httpSaveFilesFromUpload(content, boundary, saveDir, result);
          }
        }
        else
        {
          throwError(makeError("", PRIO_WARNING, ERR_SYSTEM, (int)ErrCode::UNEXPECTEDSTATE,
                               "Your device needs to be authorized - logfiles not accepted!"));
        }
      }
      else
      {
        throwError(makeError("", PRIO_WARNING, ERR_SYSTEM, (int)ErrCode::UNEXPECTEDSTATE,
                             "UUID does not exist - UUID invalid - logfiles not accepted!"));
      }
    }
  }

  //--------------------------------------------------------------------------------
  // return the config/touchscreen.css file

  static dyn_string getTouchStyleSheet()
  {
    string contents = getMergedStylesheets("touchscreen.css");

    if ( contents == "" )
      return notFoundAnswer();
    else
      return makeDynString(contents, "Status: 200 OK");
  }

  //--------------------------------------------------------------------------------
  // return the config/host-cert.pem file

  static dyn_string getHostCert(dyn_string names, dyn_string values, string user, string ip,
                                dyn_string headerNames, dyn_string headerValues)
  {
    return getFile(names, values, user, ip, headerNames, headerValues, "host-cert.pem", CONFIG_REL_PATH);
  }

  //--------------------------------------------------------------------------------
  // return the config/host-key.pem file

  static dyn_string getHostKey(dyn_string names, dyn_string values, string user, string ip,
                               dyn_string headerNames, dyn_string headerValues)
  {
    return getFile(names, values, user, ip, headerNames, headerValues, "host-key.pem", CONFIG_REL_PATH);
  }

  //--------------------------------------------------------------------------------
  // return the config/root-cert.pemfile

  static dyn_string getRootCert(dyn_string names, dyn_string values, string user, string ip,
                                dyn_string headerNames, dyn_string headerValues)
  {
    return getFile(names, values, user, ip, headerNames, headerValues, "root-cert.pem", CONFIG_REL_PATH);
  }

  //--------------------------------------------------------------------------------
  // return the file content of fileName

  static dyn_string getFile(dyn_string names, dyn_string values, string user, string ip,
                            dyn_string headerNames, dyn_string headerValues,
                            string fileName, string subPath = "")
  {
    string filePath = getPath(subPath, fileName);
    if ( filePath == "" )  // does not exist
      return notFoundAnswer();

    time m = getFileModificationTime(filePath);

    int idx = dynContains(headerNames, "If-Modified-Since");
    if ( idx > 0 )
    {
      time t = httpParseDate(headerValues[idx]);

      // to be able to have the exact same file on the client as on the server,
      // compare times as equal
      if ( m == t )
        return makeDynString("", "Status: 304 Not Modified");
    }

    string config;
    fileToString(filePath, config);
    return makeDynString(config, "Status: 200 OK", "Last-Modified: " + formatTime("HTTP", m));
  }

  //--------------------------------------------------------------------------------

  static void cbConnectUI(anytype atUserData, dyn_dyn_anytype ddaQRes)
  {
    dyn_bool unlocked;
    dyn_string dsUUIDs;
    dpGet("_UiDevices.Unlocked", unlocked,
          "_UiDevices.UUID", dsUUIDs);

    for (int i = 2; i <= dynlen(ddaQRes); i++)
    {
      mapping mTemp = jsonDecode(ddaQRes[i][2]);
      if ( mappingHasKey(mTemp, "id") )
      {
        string uuid;
        int uiManNum;
        uuid = mTemp["id"];
        sscanf(dpSubStr(ddaQRes[i][1], DPSUB_DP), "_Ui_%d", uiManNum);

        if ( !mappingHasKey(mUiUuid, uiManNum) )
          mUiUuid[uiManNum] = uuid;

        int pos = dynContains(dsUUIDs, uuid);
        if ( pos > 0 )
        {
          if ( unlocked[pos] == FALSE )
          {
            int manId = convManIdToInt(UI_MAN, uiManNum);

            if ( isRedundant() )
              dpSet("_Managers.Exit", manId, "_Managers_2.Exit", manId);
            else
              dpSet("_Managers.Exit", manId);
          }
        }
      }
    }
  }

  //--------------------------------------------------------------------------------

  static void cbDisconnectUI(anytype atUserData, dyn_dyn_anytype ddaQRes)
  {
    for (int i = 2; i <= dynlen(ddaQRes); i++)
    {
      int uiManNum;
      sscanf(dpSubStr(ddaQRes[i][1], DPSUB_DP), "_Ui_%d", uiManNum);

      if ( mappingHasKey(mUiUuid, uiManNum) )
        mappingRemove(mUiUuid, uiManNum);
    }
  }

  //--------------------------------------------------------------------------------
  /**
    @author jhercher
    callback function handling UI Managers connecting and disconnecting
  */
  static void cbHandleUiConnections(string dp1, dyn_int connectedUIs1, string dp2, dyn_int connectedUIs2)
  {
    dynAppend(connectedUIs1, connectedUIs2);
    dynUnique(connectedUIs1);

    authServerSide.handleConnections(connectedUIs1);
  }

  //--------------------------------------------------------------------------------
  /**
    @author jhercher
    callback function handling Sessiontoken written to _System.Auth.SessionTokenInterface
    this callback function must only be executed by one webserver! This is especially true on a redu system where only
    the webclient from the active system must execute this or changes to the session tokens might be lost.
  */

  static void cbHandleSessionTokens(string dp, string token)
  {
    string prefix = token[0];
    token = substr(token, 1);

    authServerSide.handleSessionTokens(prefix, token);
  }

  //--------------------------------------------------------------------------------
  /**
    @author jhercher
    function parses infoList path to see if unallowed path traversal happens
    @param path: path to be parsed
    @return bool: if path contains .. indicating parentdirectory the function returns TRUE else FALSE
    */

  static bool accessAllowed(string path)
  {
    strreplace(path, "\\", "/");
    dyn_string pathTokens = stringToDynString(path, "/");
    int sum = 0;

    for (int i = 1; i <= dynlen(pathTokens); i++)
    {
      int len = strlen(pathTokens[i]);
      if (pathTokens[i] == "..")
      {
        sum--;
      }
      else if (len > 0 && pathTokens[i] != ".")
      {
        sum++;
      }
      if (sum < 0)
      {
        return FALSE;
      }
    }

    return TRUE;
  }

  //--------------------------------------------------------------------------------

  static int httpPort, httpsPort;
  static mixed httpAuth;
  static mapping mUiUuid;
  static mixed authServerSide;
  static bool isServerSideLoginEnabled;
  static bool ulcNeedAuth;
  static int lowestAutoManNumMobileUI;
  static string projectExtension;

  static const mixed UNDEFD_AUTH;  // empty mixed
  static const string MOBILE_UI_PRAEFIX_DP = "_UiDevices";
  static const string MOBILE_UI_PRAEFIX_DP_2 = "_UiDeviceMgmt";
  static const string CONFIG_FILE = getPath(CONFIG_REL_PATH) + "config";

  static const string NOT_FOUND =
      "<html><head><title>Error</title></head>"
      "<body><h1>Not found</h1>"
      "The requested resource was not found</body></html>";

  static const float MAX_SMARTPHONE_SCREEN = 7.0;  // maximum screen diagonal for smartphone in Inches
};

//--------------------------------------------------------------------------------
