// $License: NOLICENSE
//--------------------------------------------------------------------------------
/**
  @file $relPath
  @copyright $copyright
  @author mmarchhart
*/

//--------------------------------------------------------------------------------
// Libraries used (#uses)

//--------------------------------------------------------------------------------
// Variables and Constants

//--------------------------------------------------------------------------------
/**
  Class representing an AKS Code
*/
class AKS
{
//--------------------------------------------------------------------------------
//@public members
//--------------------------------------------------------------------------------

  /** The Default Constructor.
  */
  public AKS()
  {
    _erfassungsbereich = "x";
    _streckenabschnitt = "xxx";
    _abschnitt = "xxxxxxx";
    _richtung = "x";
    _spur = "xx";
    _raeumlichkeit = "xx";
    _stockwerk = "x";
    _raum = "xxx";
    _lage = "xx";
    _anlage = "xxx";
    _anlagenteil = "xxxxx";
    _gruppe = "xxxx";
    _objekt = "xxxxxxx";
    _kodierung = "xxxxxxx";
    _funktion = "xxxxx";
  }

  public static shared_ptr<AKS> Parse(const string &aksString)
  {
    shared_ptr<AKS> aks = new AKS();

    if(aksString.length() == 53)
    {
      aks._erfassungsbereich = aksString.mid(0, 1);
      aks._streckenabschnitt = aksString.mid(1, 3);
      aks._abschnitt = aksString.mid(4, 7);
      aks._richtung = aksString.mid(11, 1);
      aks._spur = aksString.mid(12, 2);
      aks._raeumlichkeit = aksString.mid(14, 2);
      aks._stockwerk = aksString.mid(16, 1);
      aks._raum = aksString.mid(17, 3);
      aks._lage = aksString.mid(20, 2);
      aks._anlage = aksString.mid(22, 3);
      aks._anlagenteil = aksString.mid(25, 5);
      aks._gruppe = aksString.mid(30, 4);
      aks._objekt = aksString.mid(34, 7);
      aks._kodierung = aksString.mid(41, 7);
      aks._funktion = aksString.mid(48, 5);
    }
    else
    {
      throwError(makeError("bsa_framework", PRIO_WARNING, ERR_PARAM, 50, "expecting 53 chars but '" + aksString + "' has " + aksString.length() + " chars"));
    }
    return aks;
  }

  public shared_ptr<AKS> Copy()
  {
    shared_ptr<AKS> aks = new AKS();

    aks._erfassungsbereich = _erfassungsbereich;
    aks._streckenabschnitt = _streckenabschnitt;
    aks._abschnitt = _abschnitt;
    aks._richtung = _richtung;
    aks._spur = _spur;
    aks._raeumlichkeit = _raeumlichkeit;
    aks._stockwerk = _stockwerk;
    aks._raum = _raum;
    aks._lage = _lage;
    aks._anlage = _anlage;
    aks._anlagenteil = _anlagenteil;
    aks._gruppe = _gruppe;
    aks._objekt = _objekt;
    aks._kodierung = _kodierung;
    aks._funktion = _funktion;

    return aks;
  }

  public string ToString()
  {
    return _erfassungsbereich +
       _streckenabschnitt +
       _abschnitt +
       _richtung +
       _spur +
       _raeumlichkeit +
       _stockwerk +
       _raum +
       _lage +
       _anlage +
       _anlagenteil +
       _gruppe +
       _objekt +
       _kodierung +
       _funktion;
  }

  public string GetErfassungsbereich()
  {
    return _erfassungsbereich;
  }

  public string GetStreckenabschnitt()
  {
    return _streckenabschnitt;
  }

  public string GetAbschnitt()
  {
    return _abschnitt;
  }

  public string GetRichtung()
  {
    return _richtung;
  }

  public string GetSpur()
  {
    return _spur;
  }

  public string GetRaeumlichkeit()
  {
    return _raeumlichkeit;
  }

  public string GetStockwerk()
  {
    return _stockwerk;
  }

  public string GetRaum()
  {
    return _raum;
  }

  public string GetLage()
  {
    return _lage;
  }

  public string GetAnlage()
  {
    return _anlage;
  }

  public string GetAnlagenteil()
  {
    return _anlagenteil;
  }

  public string GetGruppe()
  {
    return _gruppe;
  }

  public string GetObjekt()
  {
    return _objekt;
  }

  public string GetKodierung()
  {
    return _kodierung;
  }

  public string GetFunktion()
  {
    return _funktion;
  }

  public void SetFunktion(const string &funktion)
  {
    _funktion = funktion;
  }

//--------------------------------------------------------------------------------
//@protected members
//--------------------------------------------------------------------------------

//--------------------------------------------------------------------------------
//@private members
//--------------------------------------------------------------------------------


//--------------------------------------------------------------------------------
//@private data
//--------------------------------------------------------------------------------
  private string _erfassungsbereich;
  private string _streckenabschnitt;
  private string _abschnitt;
  private string _richtung;
  private string _spur;
  private string _raeumlichkeit;
  private string _stockwerk;
  private string _raum;
  private string _lage;
  private string _anlage;
  private string _anlagenteil;
  private string _gruppe;
  private string _objekt;
  private string _kodierung;
  private string _funktion;
};
