// $License: NOLICENSE
//--------------------------------------------------------------------------------
/**
  @file $relPath
  @copyright $copyright
  @author mmarchhart
*/

//--------------------------------------------------------------------------------
// Libraries used (#uses)
#uses "classes/bsa/BleSupervisionArea"
#uses "classes/bsa/BleSupervisionUserGroup"

//--------------------------------------------------------------------------------
// Variables and Constants

//--------------------------------------------------------------------------------
/**
*/
class BleSupervision
{
//--------------------------------------------------------------------------------
//@public members
//--------------------------------------------------------------------------------

  //------------------------------------------------------------------------------
  /** The Default Constructor.
  */
  public BleSupervision(const string &rawSupervisionString)
  {
    _rawSupervisionString = rawSupervisionString;

    ParseRawString();
  }

  /**
    Checks if a given user group has currently authorization in the specified supervision area
    @param userGroup
    @param supervisionArea
    @return
  */
  public bool IsAuthorized(const BleSupervisionUserGroup &userGroup, const string &supervisionArea)
  {
    if(_supervisionAreas.contains(supervisionArea))
    {
      shared_ptr<BleSupervisionArea> area = _supervisionAreas.value(supervisionArea);
      return area.HasGroupAuthorization(userGroup);
    }
    else //mma: if the specified supervision area is not known
    {
      return false;
    }
  }

  /**
    Returns the raw supervision string as reveived from the BLE
    @return
  */
  public string GetRawSupervision()
  {
    return _rawSupervisionString;
  }


//--------------------------------------------------------------------------------
//@protected members
//--------------------------------------------------------------------------------

//--------------------------------------------------------------------------------
//@private members
//--------------------------------------------------------------------------------
  /**
    Parses the raw supervsion string into an internal structure
    The string has the format supervisionArea1@BLEgroup1|supervisionArea2@BLEgroup1@BLEgroup2| ....
    Each area is separated via a '|'
    Each area name is followed by a list of BLE groups that currently have authorization in the area. This information is separated via a '@'
  */
  public void ParseRawString()
  {
    vector<string> rawAreas = _rawSupervisionString.split("|");  //mma: split the string into the individual areas

    for(int i=0 ; i < rawAreas.count() ; ++i)
    {
      string rawArea = rawAreas.at(i);

      vector<string> areaSplit = rawArea.split("@");  //mma: split the area string to get the areaName and the ble groups that are currently authorized

      if(areaSplit.count() >= 1)
      {
        string areaName = areaSplit.at(0);  //mma: the first element is are supervision area name

        vector<BleSupervisionUserGroup> authorizedGroups;

        for(int j=1 ; j < areaSplit.count() ; ++j)  //mma: loop throgh all authorized groups and parse them into the internal class represnentation
        {
          BleSupervisionUserGroup supervisionGroup = BleSupervisionUserGroup::FromString(areaSplit.at(j));

          if(supervisionGroup.GetUserGroupEnum() != BleUserGroup::NONE)  //mma: only append if it is a proper group. E.g. if no group has authroization the BLE puts root as group in the supervision string
          {
            authorizedGroups.append(supervisionGroup);
          }
        }

        shared_ptr<BleSupervisionArea> supervisionArea = new BleSupervisionArea(areaName);
        supervisionArea.SetAuthorizedGroups(authorizedGroups);

        _supervisionAreas.insert(areaName, supervisionArea);  //mma: add the whole supervision area to the internal mapping
      }
    }
  }

//--------------------------------------------------------------------------------
//@private data
//--------------------------------------------------------------------------------
  private string _rawSupervisionString;
  private mapping _supervisionAreas;
};
