// $License: NOLICENSE
//--------------------------------------------------------------------------------
/**
  @file $relPath
  @copyright BECC GmbH
  @author mmarchhart
*/

//--------------------------------------------------------------------------------
// Libraries used (#uses)

//--------------------------------------------------------------------------------
// Variables and Constants

//--------------------------------------------------------------------------------
/**
*/
class BsaDpGroupCache
{
//--------------------------------------------------------------------------------
//@public members
//--------------------------------------------------------------------------------

  //------------------------------------------------------------------------------
  /** The Default Constructor.
  */
  public BsaDpGroupCache()
  {
    _isDpGroupInit=false;
  }

  /**
    Initializes the dpGroup cache
  */
  public void InitDPGroupCache()
  {
    if(!_isDpGroupInit)
    {
      //connections for the creation/updating of the dpGroup cache
      dpQueryConnectSingle(this, dpGroupCB, true, "", "SELECT '_original.._value' FROM '{*.Types*,*.Dps*}' WHERE _DPT =  \"_DpGroup\"");
      sysConnect(this, dpCreatedCB, "dpCreated");
      sysConnect(this, dpDeletedCB, "dpDeleted");

      while(!_isDpGroupInit)
      {
        delay(0,500);  //wait till the dpGroup cache is initialized
      }
    }
  }

  /**
    Searches for the DPGroup to which the given DPE belongs. In case of no match, an empty string is returned.
    @param dpe in the format systemName:DP.dpe
    @return name of the DPGroup to which the given dpe belongs to
  */
  public string FindDPEGroup(const string &dpe)
  {
    synchronized(_dpGroupMutex)
    {
      dyn_string keys = mappingKeys(_mDPGroups);

      for(int i=1 ; i<=keys.count() ; ++i)
      {
        if(dynContains(_mDPGroups[keys[i]],dpe))
        {
          return keys[i];
        }
      }

      return "";  //mma: returns empty string if given dpe is not contained in any group
    }
  }

//--------------------------------------------------------------------------------
//@protected members
//--------------------------------------------------------------------------------

//--------------------------------------------------------------------------------
//@private members
//--------------------------------------------------------------------------------
  private void dpGroupCB(string s, dyn_dyn_anytype ddaTab)
  {
    refreshDPGroupCache();
  }

  private void dpCreatedCB(string event, mapping data)
  {
    if(data["dpType"] != "_DpGroup")
    {
      refreshDPGroupCache();  //the refresh on a new created dpGroup happens via dpGroupCB once the filters of the new group are set
    }
  }

  private void dpDeletedCB(string event, mapping data)
  {
    refreshDPGroupCache();
  }

  private void refreshDPGroupCache()
  {
    mapping mTemp;
    dyn_string dstrGroupDPs = groupGetDps();

    for(int i=1 ; i<=dynlen(dstrGroupDPs); ++i)
    {
      dyn_string res_dpfilter, res_typfilter;
      string strGroupName = groupDpNameToName(dstrGroupDPs[i]);
      mTemp[strGroupName] = makeDynString();

      groupGetFilterItems(dstrGroupDPs[i], res_typfilter, res_dpfilter);

      for(int j=1 ; j<=dynlen(res_typfilter) ; ++j)
      {
        dyn_string dsItemsInFilter;
        int overflow;

        groupGetFilteredDps(res_typfilter[j], res_dpfilter[j], dsItemsInFilter, overflow);
        dynAppend(mTemp[strGroupName],dsItemsInFilter);
      }
    }

    synchronized(_dpGroupMutex)
    {
      _mDPGroups = mTemp;
    }

    _isDpGroupInit=true;
  }

//--------------------------------------------------------------------------------
//@private data
//--------------------------------------------------------------------------------
  private mapping _mDPGroups;
  private bool _dpGroupMutex;
  private bool _isDpGroupInit;

};
