// $License: NOLICENSE
//--------------------------------------------------------------------------------
/**
  @file $relPath
  @copyright BECC GmbH
  @author
*/

//--------------------------------------------------------------------------------
// used libraries (#uses)

//--------------------------------------------------------------------------------
// variables and constants

//--------------------------------------------------------------------------------
/**
*/
enum BsaPanelLoadingType
{
  STANDARD,
  DETAIL,
  LOAD_FIRST_DETAIL
};

struct BsaPanelNavDetail
{
  string CNSPath;
  string ID;
  langString PanelName;
  string PanelFile;
  string IconFile;
  string Description;
  BsaPanelLoadingType LoadingType;
};

class BsaPanelNavigation
{
//--------------------------------------------------------------------------------
//@public members
//--------------------------------------------------------------------------------

  //------------------------------------------------------------------------------
  /** Default constructor.
  */
  public BsaPanelNavigation()
  {

  }

  public static string saveNavDetails(string id, mapping txt)
  {
    string sNewId = txt["ID"];

    if ( nameCheck(sNewId) != 0 )
    {
      ChildPanelOnCentral("vision/MessageWarning", "Warnung", makeDynString("Die Knoten-ID beinhaltet nicht erlaubte Zeichen.\nDie Änderungen wurden nicht übernommen"));
      return "";
    }

    string sNewPath = id;

    if ( sNewId != cnsSubStr(id, CNSSUB_NODE) )//id aender sich
    {
      if ( cns_isTree(id) )
      {
        strreplace(sNewPath, ":"+cnsSubStr(id, CNSSUB_NODE), ":"+sNewId);

        if ( cns_treeExists(sNewPath) )
        {
          ChildPanelOnCentral("vision/MessageWarning", "Warnung", makeDynString("Die Knoten-ID existiert bereits.\nDie Änderungen wurden nicht übernommen"));
          return "";
        }
      }

      if ( cns_isNode(id) )
      {
        strreplace(sNewPath, "."+cnsSubStr(id, CNSSUB_NODE), "."+sNewId);

        if ( cns_nodeExists(sNewPath) )
        {
          ChildPanelOnCentral("vision/MessageWarning", "Warnung", makeDynString("Die Knoten-ID existiert bereits.\nDie Änderungen wurden nicht übernommen"));
          return "";
        }
      }
    }

    string sDp;

    cnsGetId(id, sDp);

    dpSet(sDp+".panelFile", txt["PANEL"],
          sDp+".icon", txt["ICON"],
          sDp+".loadingType", txt["LOADINGTYPE"],
          sDp+".panelDescription", txt["DESCRIPTION"]);

    cnsChangeNodeDisplayNames(id, txt["NAME"]);

    cnsChangeNodeName(id, sNewId);
    delay(0,250);
    return sNewPath;
  }

  public static string createNavNode(string sParentId)
  {
    string sSystem = getSystemName();
    string sNewNodeIdRet;
    string sNewNodeId;

    strreplace(sSystem, ":", "");

    if ( sParentId == "" || sParentId == "0" )
      sParentId = sSystem+"._BsaNavigation:";

    dyn_string dsChildren;

    if ( cns_isView(sParentId) )
      cnsGetTrees(sParentId, dsChildren);
    else
      cnsGetChildren(sParentId, dsChildren);

    langString sNewNodeName = "Neuer Knoten";

    int i = 1;

    while ( true )
    {
      sNewNodeId = i;

      string sTemp = sParentId + (cns_isView(sParentId)?"":".") + sNewNodeId;

      if ( !dynContains(dsChildren, sTemp) )
      {
        sNewNodeName += " "+i;
        break;
      }
      i++;
    }

    int iOrder = 0;

    for ( int i = 1; i <= dynlen(dsChildren); i++ )
    {
      int iTemp;

      dpGet(dsChildren[i]+".:_general.._int_01", iTemp);

      if ( iTemp > iOrder )
        iOrder = iTemp;
    }

    iOrder++;

    dyn_string dsDps = dpNames("*", "_BsaNavigationNode");
    string sDpName;

    i = 1;

    while ( true )
    {
      string sTemp = "_BsaNavigationNode_"+i;

      if ( !dynContains(dsDps, getSystemName()+sTemp) )
      {
        sDpName = sTemp;
        break;
      }
      i++;
    }

    if ( cns_isView(sParentId) )
    {
      sNewNodeIdRet = sParentId+""+sNewNodeId;
    }
    else
    {
      sNewNodeIdRet = sParentId+"."+sNewNodeId;
    }

    if ( !dpExists(sDpName) )
    {
      dpCreate(sDpName, "_BsaNavigationNode");

      dpSetWait(sDpName+".:_general.._type", DPCONFIG_GENERAL,
                sDpName+".:_general.._int_01", iOrder);
    }

    cns_createTreeOrNode(sParentId, sNewNodeId, sNewNodeName, sDpName, CNS_DATATYPE_DATAPOINT);
    return sNewNodeIdRet;
  }

  public static void deleteNavNode (string sId)
  {
    dyn_string ds;
    dyn_float df;

    string sMsg = getCatStr("bsa_framework", "securityQueryDelNav");
    string sYes = getCatStr("bsa_framework", "yes");
    string sNo = getCatStr("bsa_framework", "no");


    if(sId == "")
      return;

    ChildPanelOnCentralModalReturn("vision/MessageInfo.pnl", "Info", makeDynString(sMsg, sYes, sNo), df, ds);

    if ( dynlen(df) < 1 )
      return;
    if ( df[1] == 0 )
      return;

    string sDp;
    dyn_string dsDps;

    cnsGetId(sId, sDp);

    dynAppend(dsDps, sDp);

    dyn_string dsChilds;

    cnsGetNodesByName(sId+".**", CNS_SEARCH_NAME, CNS_SEARCH_ALL_LANGUAGES, CNS_DATATYPE_ALL_TYPES, dsChilds);

    for ( int i = 1; i <= dynlen(dsChilds); i++ )
    {
      cnsGetId(dsChilds[i], sDp);
      dynAppend(dsDps, sDp);
    }

    bool bRet = cns_deleteTreeOrNode(sId);

    for ( int i = 1; i <= dynlen(dsDps); i++ )
      dpDelete(dsDps[i]);
  }

  public static BsaPanelNavDetail getNavDetails(string cnsPath)
  {
    BsaPanelNavDetail navDetail;

    navDetail.CNSPath = cnsPath;
    navDetail.ID = cnsSubStr(cnsPath, CNSSUB_NODE);

    langString lsName;

    cnsGetDisplayNames(cnsPath, lsName);

    navDetail.PanelName = lsName;

    string sDp;

    cnsGetId(cnsPath, sDp);

    string sPanel;
    string sIcon;
    int iLoadingType;
    string description;

    if(sDp != "")
    {
      dpGet(sDp+".panelFile", sPanel,
            sDp+".icon", sIcon,
            sDp+".loadingType", iLoadingType,
            sDp+".panelDescription", description);
    }

    navDetail.PanelFile = sPanel;
    navDetail.IconFile = sIcon;
    navDetail.LoadingType = iLoadingType;
    navDetail.Description = description;

    return navDetail;
  }

  public static void generateSumAlerts()
  {
    string sSystem = getSystemName();

    strreplace(sSystem, ":", "");

    string sViewName = sSystem + "._BsaNavigation:";
    dyn_string dsAllNodes;

    cnsGetNodesByName("*", sViewName, CNS_SEARCH_NAME, CNS_SEARCH_ALL_LANGUAGES, CNS_DATATYPE_ALL_TYPES, dsAllNodes);

    if (dynlen(dsAllNodes)<1)
      return;

    progressBar.maximum(0);
    progressBar.maximum(dynlen(dsAllNodes));

    for ( int i = 1; i <= dynlen(dsAllNodes); i++ )//generate all sum alerts
    {
      progressBar.value = i;

      string sPanelname;
      string sNaviagationDp;

      cnsGetId(dsAllNodes[i], sNaviagationDp);

      if ( !dpExists(sNaviagationDp) )
      {
        throwError(makeError("bsa_framework", PRIO_WARNING, ERR_IMPL, 10, "("+dsAllNodes[i]+"): "+sNaviagationDp));
        continue;
      }

      dpGet(sNaviagationDp + ".panelFile", sPanelname);

      if ( !isfile(getPath(PANELS_REL_PATH, sPanelname)) && sPanelname != "" )
      {
        throwError(makeError("bsa_framework", PRIO_WARNING, ERR_PARAM, 11, "("+dsAllNodes[i]+"): "+sPanelname));

        DeleteAlertHdl(sNaviagationDp + ".sumAlerts");
        DeleteAlertHdl(sNaviagationDp + ".sumAlerts.alert");
        DeleteAlertHdl(sNaviagationDp + ".sumAlerts.fault");

        continue;
      }

      dyn_string dsList;

      if ( sPanelname != "" )
        getConfirmDps(sPanelname, dsList);

      dyn_string dsAlertList;
      dyn_string dsAlertDpes;

      for (int i = 1; i <= dynlen(dsList); i++)
      {
        if(patternMatch("*|*",dsList[i]))
        {
          dyn_string dsTemp = stringToDynString(dsList[i]);
          for (int j = 1; j <= dynlen(dsTemp); j++)
          {
            dynAppend(dsAlertList, dsTemp[j]);
          }
        }
        if(dsList[i] != "")
          dynAppend(dsAlertList, dsList[i]);
      }

      dyn_string objectDps;

      //dsAlertList run through to check if dp exists
      for ( int j = dynlen(dsAlertList); j > 0; j-- )
      {
        //check if dp exists
        if ( !dpExists(dsAlertList[j]) )
        {
          throwError(makeError("bsa_framework", PRIO_WARNING, ERR_PARAM, 12, "("+dsAllNodes[i]+") '"+dsAlertList[j]+"' in panel: "+sPanelname));
          dynRemove(dsAlertList, j);
        }
        else
        {
          if(strpos(dsAlertList[j],".")>0) //If DP has an element
          {
            dynAppend(dsAlertDpes, dsAlertList[j]);
            objectDps.append(dpSubStr(dsAlertList[j], DPSUB_DP));
          }
          else
          {
            dyn_string dsTemp = dpNames(dsAlertList[j]+".**");
            dynAppend(dsAlertDpes, dsTemp);

            objectDps.append(dsAlertList[j]);
          }
        }
      }

      objectDps.unique();

      dyn_string dsAlertOnlyDpes;
      //loop through all dpes to get only those with an configured alert handler
      if(dynlen(dsAlertDpes)>0)
      {
        dyn_string dsAlertTypeConfig;

        for(int j=1 ; j<=dynlen(dsAlertDpes) ; ++j)
        {
          dynAppend(dsAlertTypeConfig, dsAlertDpes[j]+":_alert_hdl.._type");
        }

        dyn_int diAlertType;
        dpGet(dsAlertTypeConfig,diAlertType);

        for(int j=1 ; j<=dynlen(dsAlertDpes) ; ++j)
        {
          if(diAlertType[j] != DPCONFIG_NONE) dynAppend(dsAlertOnlyDpes,dsAlertDpes[j]);  //append those with an configured alert handler
        }
      }

      dsAlertDpes = dsAlertOnlyDpes;

      //add child nodes
      dyn_string dsChildNodes;
      dyn_string childNavigationDps;

      cnsGetChildren(dsAllNodes[i], dsChildNodes);

      for ( int j = 1; j <= dynlen(dsChildNodes); j++ )//get sub alerts
      {
        string sTempDp;

        cnsGetId(dsChildNodes[j], sTempDp);

        if ( dpExists(sTempDp) )
        {
          dynAppend(dsAlertDpes, sTempDp + ".sumAlerts");
          childNavigationDps.append(sTempDp);
        }
      }

      //sumalert start
      DeleteAlertHdl(sNaviagationDp + ".sumAlerts");

      //create alertConfig
      SetSumAlertHdl(sNaviagationDp + ".sumAlerts", makeDynString(sNaviagationDp + ".sumAlerts.alert", sNaviagationDp + ".sumAlerts.fault"), "");

      //sumalert end

      //alert start
      DeleteAlertHdl(sNaviagationDp + ".sumAlerts.alert");

      //create alertConfig
      dynStrReplace(dsAlertDpes,".sumAlerts",".sumAlerts.alert");
      SetSumAlertHdl(sNaviagationDp + ".sumAlerts.alert", dsAlertDpes, "90-255");

      //alert end

      //fault start
      DeleteAlertHdl(sNaviagationDp + ".sumAlerts.fault");

      //create alertConfig
      dynStrReplace(dsAlertDpes,".sumAlerts.alert",".sumAlerts.fault");
      SetSumAlertHdl(sNaviagationDp + ".sumAlerts.fault", dsAlertDpes, "60-89");

      //fault end

      //Create the sum alert handler for mode_serve ("Bedienart")
      SetSumServeHdl(sNaviagationDp, childNavigationDps, objectDps);

      //Create the sum alert handler for mode_control ("Steuerungsart")
      SetSumControlHdl(sNaviagationDp, childNavigationDps, objectDps);

      throwError(makeError("bsa_framework", PRIO_INFO, ERR_IMPL, 13, dsAllNodes[i]));
    }

  }

  /**
    Returns the configured panel loading type of a given cnsPanelPath
    @param cnsPanelPath
    @return
  */
  public static BsaPanelLoadingType GetLoadingType(string cnsPanelPath)
  {
    int loadingType;

    dpGet(cnsPanelPath + ".loadingType", loadingType);

    BsaPanelLoadingType panelLoadingType = loadingType;

    return panelLoadingType;
  }

  /**
    Returns the cnsPanelPath of the panel defined on the left side of the given path (at the same hierarchy in the topology).
    In case the given path is already the most left one an empty string is returned.
    @param cnsPanelPath
    @return
  */
  public static string GetLeftCNSPanelPath(string cnsPanelPath)
  {
    string leftCnsPanelPath;
    string parentCnsPath;

    if(cnsGetParent(cnsPanelPath, parentCnsPath))
    {
      dyn_string siblingPaths;

      if(cnsGetChildren(parentCnsPath, siblingPaths))
      {
        int ownIndex = siblingPaths.indexOf(cnsPanelPath);

        if(ownIndex > 0)
        {
          leftCnsPanelPath = siblingPaths.at(ownIndex - 1);
        }
      }
    }

    return leftCnsPanelPath;
  }

  /**
    Returns the cnsPanelPath of the panel defined on the right side of the given path (at the same hierarchy in the topology).
    In case the given path is already the most right one an empty string is returned.
    @param cnsPanelPath
    @return
  */
  public static string GetRightCNSPanelPath(string cnsPanelPath)
  {
    string rightCnsPanelPath;
    string parentCnsPath;

    if(cnsGetParent(cnsPanelPath, parentCnsPath))
    {
      dyn_string siblingPaths;

      if(cnsGetChildren(parentCnsPath, siblingPaths))
      {
        int ownIndex = siblingPaths.indexOf(cnsPanelPath);

        if(ownIndex > -1 && ownIndex < (siblingPaths.count() - 1))
        {
          rightCnsPanelPath = siblingPaths.at(ownIndex + 1);
        }
      }
    }

    return rightCnsPanelPath;
  }

//--------------------------------------------------------------------------------
//@protected members
//--------------------------------------------------------------------------------

//--------------------------------------------------------------------------------
//@private members
//--------------------------------------------------------------------------------

  private static void dynStrReplace(dyn_string &dsSource, string sSearch, string sReplace)
  {
    for (int i = 1; i <= dynlen(dsSource); i++)
    {
      strreplace(dsSource[i],sSearch,sReplace);
    }
  }

  private static dyn_string dynStrAdd(const dyn_string &original, const string &append)
  {
    dyn_string result;

    for(int i=0 ; i < original.count() ; ++i)
    {
      result.append(original.at(i) + append);
    }

    return result;
  }

  /**
    Deletes the alert hdl config from the given datapoint element
    @param dpe
  */
  private static void DeleteAlertHdl(const string &dpe)
  {
    dpSetTimedWait(0, dpe + ":_alert_hdl.._ack", 1);
    dpSetTimedWait(0, dpe + ":_alert_hdl.._active", false);
    dpSetTimedWait(0, dpe + ":_alert_hdl.._type", DPCONFIG_NONE);//loeschen
  }

  private static void SetSumAlertHdl(const string &dpe, const dyn_string &dpList, const string &prioPattern)
  {
    dpSetTimedWait(0, dpe + ":_alert_hdl.._type", DPCONFIG_SUM_ALERT);

    dpSetTimedWait(0, dpe + ":_alert_hdl.._class", "",
                      dpe + ":_alert_hdl.._help", "",
                      dpe + ":_alert_hdl.._dp_list", dpList,
                      dpe + ":_alert_hdl.._ack_has_prio", true,    // unquittiert hat Prioritaet
                      dpe + ":_alert_hdl.._order", 2,
                      dpe + ":_alert_hdl.._prio_pattern", prioPattern);

    dpSetTimedWait(0, dpe + ":_alert_hdl.._active", true);
  }

  private static void SetSumServeHdl(const string &navigationDp, const dyn_string &childNavigationDps, const dyn_string &objectDps)
  {
    string sumServLocalDpe    = ".sumServe.local";
    string sumServRevisionDpe = ".sumServe.revision";
    string sumServInvalidDpe  = ".sumServe.invalid";

    DeleteAlertHdl(navigationDp + sumServLocalDpe);
    DeleteAlertHdl(navigationDp + sumServRevisionDpe);
    DeleteAlertHdl(navigationDp + sumServInvalidDpe);

    dyn_string objectDpList;

    for(int i=0 ; i < objectDps.count() ; ++i)
    {
      string objectModeServeDpe = objectDps.at(i) + ".cmdio.mode_serve";

      if(dpExists(objectModeServeDpe))  //mma: add it only to the list if the object hast the mode_serve dpe
      {
        objectDpList.append(objectModeServeDpe);
      }
    }

    dyn_string dpListLocal = objectDpList;
    dynAppend(dpListLocal, dynStrAdd(childNavigationDps, sumServLocalDpe));
    SetSumAlertHdl(navigationDp + sumServLocalDpe, dpListLocal, "7");

    dyn_string dpListRevision = objectDpList;
    dynAppend(dpListRevision, dynStrAdd(childNavigationDps, sumServRevisionDpe));
    SetSumAlertHdl(navigationDp + sumServRevisionDpe, dpListRevision, "8");

    dyn_string dpListInvalid = objectDpList;
    dynAppend(dpListInvalid, dynStrAdd(childNavigationDps, sumServInvalidDpe));
    SetSumAlertHdl(navigationDp + sumServInvalidDpe, dpListInvalid, "9");
  }

  private static void SetSumControlHdl(const string &navigationDp, const dyn_string &childNavigationDps, const dyn_string &objectDps)
  {
    string sumControlManDpe    = ".sumControl.man";
    string sumControlProgDpe = ".sumControl.program";
    string sumControlManTempDpe  = ".sumControl.manTemp";

    DeleteAlertHdl(navigationDp + sumControlManDpe);
    DeleteAlertHdl(navigationDp + sumControlProgDpe);
    DeleteAlertHdl(navigationDp + sumControlManTempDpe);

    dyn_string objectDpList;

    for(int i=0 ; i < objectDps.count() ; ++i)
    {
      string objectModeServeDpe = objectDps.at(i) + ".cmdio.mode_control";

      if(dpExists(objectModeServeDpe))  //mma: add it only to the list if the object hast the mode_control dpe
      {
        objectDpList.append(objectModeServeDpe);
      }
    }

    dyn_string dpListMan = objectDpList;
    dynAppend(dpListMan, dynStrAdd(childNavigationDps, sumControlManDpe));
    SetSumAlertHdl(navigationDp + sumControlManDpe, dpListMan, "7");

    dyn_string dpListProg = objectDpList;
    dynAppend(dpListProg, dynStrAdd(childNavigationDps, sumControlProgDpe));
    SetSumAlertHdl(navigationDp + sumControlProgDpe, dpListProg, "8");

    dyn_string dpListManTemp = objectDpList;
    dynAppend(dpListManTemp, dynStrAdd(childNavigationDps, sumControlManTempDpe));
    SetSumAlertHdl(navigationDp + sumControlManTempDpe, dpListManTemp, "9");
  }
};
