// $License: NOLICENSE
//--------------------------------------------------------------------------------
/**
  @file $relPath
  @copyright $copyright
  @author mmarchhart
*/

//--------------------------------------------------------------------------------
// Libraries used (#uses)
#uses "classes/bsa/AKS"
#uses "classes/bsa/BsaObject"
#uses "classes/bsa/objectFactory/BsaObjectFactoryInput"

//--------------------------------------------------------------------------------
// Variables and Constants

//--------------------------------------------------------------------------------
/**
*/
class BsaObjectFactory
{
//--------------------------------------------------------------------------------
//@public members
//--------------------------------------------------------------------------------

  //------------------------------------------------------------------------------
  /** The Default Constructor.
  */
  public BsaObjectFactory()
  {
  }

//--------------------------------------------------------------------------------
//@protected members
//--------------------------------------------------------------------------------
  /**
    Creates the datapoint with the given name of the given dpt and sets all required information for a BsaObject (like alert handers for mode_serve and mode_control)
    Throws an exception in case the datapoint couldn't be created
    @param objectDpName
    @param dpType
  */
  protected static void CreateBsaObjectDP(shared_ptr<BsaObjectFactoryInput> objectInput, const string &dpType)
  {
    if(dpCreate(objectInput.GetObjectDpName(), dpType) ==0)
    {
      EnableGeneralConfigs(objectInput.GetObjectDpName());
      SetModeServeAlHdl(objectInput.GetObjectDpName());
      SetModeControlAlHdl(objectInput.GetObjectDpName());
      SetSpezAlHdl(objectInput.GetObjectDpName());

      SetBaseObjectAKS(objectInput);
    }
    else
    {
      dyn_errClass error = getLastError();
      throw(makeError("bsaObject", PRIO_SEVERE, ERR_PARAM, 1, error));
    }
  }

  /**
    Deletes the alert hdl config from the given datapoint element
    @param dpe
  */
  protected static void DeleteAlertHdl(const string &dpe)
  {
    dpSetTimedWait(0, dpe + ":_alert_hdl.._ack", 1);
    dpSetTimedWait(0, dpe + ":_alert_hdl.._active", false);
    dpSetTimedWait(0, dpe + ":_alert_hdl.._type", DPCONFIG_NONE);//loeschen
  }

  protected static void SetBoolAlert(const string &dpe, const string &cameText, const string &alertClass)
  {

    //mma: assumtion is here that value false is the good range!!!
    dpSetTimedWait(0, dpe+":_alert_hdl.._type", DPCONFIG_ALERT_BINARYSIGNAL,
                      dpe+":_alert_hdl.._text1", cameText,
                      dpe+":_alert_hdl.._class", alertClass+".",
                      dpe+":_alert_hdl.._ok_range", false,
                      dpe+":_alert_hdl.._active", true);
  }

  protected static void AddDiscreteAlertRange(dyn_string &details, dyn_mixed &values, const string &dpe, int rangeNumber, int value, const string &cameText, const string &alertClass)
  {
    string matchValue;
    string alClass = alertClass;

    if(alClass != "")
    {
      alClass+=".";  //mma: add the required "." behinde the alert class name
      matchValue = value;
    }
    else   //mma: if no class is defined it means it is the good range, therefor we set the matchValue to "*"
    {
      matchValue = "*";
    }
    //DebugTN("Set Range:" + rangeNumber + " " + value + " " + alClass);

    details.append(dpe + ":_alert_hdl."+rangeNumber+"._type");
    values.append(DPDETAIL_RANGETYPE_MATCH);

    details.append(dpe + ":_alert_hdl."+rangeNumber+"._match");
    values.append(matchValue);

    details.append(dpe + ":_alert_hdl."+rangeNumber+"._text");
    values.append(cameText);

    details.append(dpe + ":_alert_hdl."+rangeNumber+"._class");
    values.append(alClass);
  }

//--------------------------------------------------------------------------------
//@private members
//--------------------------------------------------------------------------------
  private static void SetModeServeAlHdl(const string &objectDpName)
  {
    string modeServeDpe = objectDpName + ".cmdio.mode_serve";

    if(dpExists(modeServeDpe))
    {
      DeleteAlertHdl(modeServeDpe);  //mma: first delete the current alert hdl to cleanly define the new one (this is relevant for the usecase of updating the object)

      dpSetTimedWait(0, modeServeDpe + ":_alert_hdl.._type", DPCONFIG_ALERT_NONBINARYSIGNAL);

      dyn_string details;
      dyn_mixed values;

      AddDiscreteAlertRange(details, values, modeServeDpe, 1, 0, "", "");  //mma: add the good range
      AddDiscreteAlertRange(details, values, modeServeDpe, 2, (int)BsaModeServe::LOCAL, getCatStr("bsaObject", "serveAlhLocal"), "BSA_MSLocal");
      AddDiscreteAlertRange(details, values, modeServeDpe, 3, (int)BsaModeServe::REVISION, getCatStr("bsaObject", "serveAlhRevision"), "BSA_MSRevision");
      AddDiscreteAlertRange(details, values, modeServeDpe, 4, (int)BsaModeServe::INVALID, getCatStr("bsaObject", "serveAlhInvalid"), "BSA_MSInvalid");

      dpSetTimedWait(0, details, values);

      dpSetTimedWait(0, modeServeDpe + ":_alert_hdl.._active", true);  //mma: activate now the alert hdl
    }
  }

  private static void SetModeControlAlHdl(const string &objectDpName)
  {
    string modeControlDpe = objectDpName + ".cmdio.mode_control";

    if(dpExists(modeControlDpe))
    {
      DeleteAlertHdl(modeControlDpe);  //mma: first delete the current alert hdl to cleanly define the new one (this is relevant for the usecase of updating the object)

      dpSetTimedWait(0, modeControlDpe + ":_alert_hdl.._type", DPCONFIG_ALERT_NONBINARYSIGNAL);

      dyn_string details;
      dyn_mixed values;

      AddDiscreteAlertRange(details, values, modeControlDpe, 1, 0, "", "");  //mma: add the good range
      AddDiscreteAlertRange(details, values, modeControlDpe, 2, (int)BsaModeControl::MANUAL, getCatStr("bsaObject", "controlAlhManual"), "BSA_MCManual");
      AddDiscreteAlertRange(details, values, modeControlDpe, 3, (int)BsaModeControl::PROGRAM, getCatStr("bsaObject", "controlAlhProgram"), "BSA_MCProgram");
      AddDiscreteAlertRange(details, values, modeControlDpe, 4, (int)BsaModeControl::MANUAL_TEMP, getCatStr("bsaObject", "controlAlhManualTemp"), "BSA_MCManualTemp");

      dpSetTimedWait(0, details, values);

      dpSetTimedWait(0, modeControlDpe + ":_alert_hdl.._active", true);  //mma: activate now the alert hdl
    }
  }

  private static void SetSpezAlHdl(const string &objectDpName)
  {
    string spezDpe = objectDpName + ".cmdio.spez";

    if(dpExists(spezDpe))
    {
      DeleteAlertHdl(spezDpe);  //mma: first delete the current alert hdl to cleanly define the new one (this is relevant for the usecase of updating the object)

      dpSetTimedWait(0, spezDpe + ":_alert_hdl.._type", DPCONFIG_ALERT_NONBINARYSIGNAL);

      dyn_string details;
      dyn_mixed values;

      AddDiscreteAlertRange(details, values, spezDpe, 1, 0, "", "");  //mma: add the good range
      AddDiscreteAlertRange(details, values, spezDpe, 2, (int)BsaSpez::BLOCKED, getCatStr("bsaObject", "spezAlhBlocked"), "BSA_SPEZBlocked");
      AddDiscreteAlertRange(details, values, spezDpe, 3, (int)BsaSpez::INACTIVE, getCatStr("bsaObject", "spezAlhInactive"), "BSA_SPEZInactive");
      AddDiscreteAlertRange(details, values, spezDpe, 4, (int)BsaSpez::WINTER, getCatStr("bsaObject", "spezAlhWinter"), "BSA_SPEZWinter");
      AddDiscreteAlertRange(details, values, spezDpe, 5, (int)BsaSpez::PRIORITY, getCatStr("bsaObject", "spezAlhPriority"), "BSA_SPEZPriority");

      dpSetTimedWait(0, details, values);

      dpSetTimedWait(0, spezDpe + ":_alert_hdl.._active", true);  //mma: activate now the alert hdl
    }
  }

  /**
    Enables the _general config on the object datapoint itself
    @param objectDpName
  */
  private static void EnableGeneralConfigs(const string &objectDpName)
  {
    dpSet(objectDpName + ".:_general.._type", DPCONFIG_GENERAL);
  }

  private static void SetBaseObjectAKS(shared_ptr<BsaObjectFactoryInput> objectInput)
  {
    string objectDpName = objectInput.GetObjectDpName();
    shared_ptr<AKS> objectAks = objectInput.GetObjectAKS();

    dpSetAlias(objectDpName + ".", objectAks.ToString());

    string modeServeDpe = objectDpName + ".cmdio.mode_serve";
    if(dpExists(modeServeDpe))
    {
      dpSetAlias(modeServeDpe, objectInput.GetModeServeAKSasString());
    }

    string modeControlDpe = objectDpName + ".cmdio.mode_control";
    if(dpExists(modeControlDpe))
    {
      dpSetAlias(modeControlDpe, objectInput.GetModeControlAKSasString());
    }

    string spezDpe = objectDpName + ".cmdio.spez";
    if(dpExists(spezDpe))
    {
      dpSetAlias(spezDpe, objectInput.GetSpezAKSasString());
    }
  }
};
