// $License: NOLICENSE
//--------------------------------------------------------------------------------
/**
  @file $relPath
  @copyright $copyright
  @author mmarchhart
*/

//--------------------------------------------------------------------------------
// Libraries used (#uses)
#uses "classes/bsa/AKS"
//--------------------------------------------------------------------------------
// Variables and Constants

//--------------------------------------------------------------------------------
/**
*/
class BsaObjectFactoryInput
{
//--------------------------------------------------------------------------------
//@public members
//--------------------------------------------------------------------------------

  //------------------------------------------------------------------------------
  /** The Default Constructor.
  */
  public BsaObjectFactoryInput(shared_ptr<AKS> objectAKS)
  {
    _objectAKS = objectAKS;
    _modeServeAKS = objectAKS.Copy();
    _modeServeAKS.SetFunktion("BEDAR");

    _modeControlAKS = objectAKS.Copy();
    _modeControlAKS.SetFunktion("STEAR");

    _spezAKS = objectAKS.Copy();
    _spezAKS.SetFunktion("SPEZx");

    _objectDpName = GenerateObjectDpName();
  }

  /**
    Returns the object dp name which was generated based on the given objectAKS
    @return
  */
  public string GetObjectDpName()
  {
    return _objectDpName;
  }

   public shared_ptr<AKS> GetObjectAKS()
  {
    return _objectAKS;
  }

  public void SetModeServeAKS(shared_ptr<AKS> aks)
  {
    assignPtr(_modeServeAKS, aks);
  }

  public shared_ptr<AKS> GetModeServeAKS()
  {
    return _modeServeAKS;
  }

  public void SetModeControlAKS(shared_ptr<AKS> aks)
  {
    assignPtr(_modeControlAKS, aks);
  }

  public shared_ptr<AKS> GetModeControlAKS()
  {
    return _modeControlAKS;
  }

  public void SetSpezAKS(shared_ptr<AKS> aks)
  {
    assignPtr(_spezAKS, aks);
  }

  public shared_ptr<AKS> GetSpezAKS()
  {
    return _spezAKS;
  }

  public string GetModeServeAKSasString()
  {
    return _modeServeAKS.ToString();
  }

  public string GetModeControlAKSasString()
  {
    return _modeControlAKS.ToString();
  }

  public string GetSpezAKSasString()
  {
    return _spezAKS.ToString();
  }

//--------------------------------------------------------------------------------
//@protected members
//--------------------------------------------------------------------------------
  /**
    Generates the object dp name based on the given objectAKS
    @return
  */
  protected string GenerateObjectDpName()
  {
    string objectDpName;

    string erfBer = _objectAKS.GetErfassungsbereich();
    erfBer.replace("x", "");

    string strAbs = _objectAKS.GetStreckenabschnitt();
    strAbs.replace("x", "");

    string ort    = _objectAKS.GetAbschnitt();
    ort.replace("x", "");

    string richtung = _objectAKS.GetRichtung();
    richtung.replace("x", "");

    string gruppe = _objectAKS.GetGruppe();
    gruppe.replace("x", "");

    string object = _objectAKS.GetObjekt();
    object.replace("x", "");

    objectDpName += erfBer + "_";
    if(strAbs.length() != 0) objectDpName += strAbs + "_";
    if(ort.length() != 0) objectDpName += ort + "_";
    if(richtung.length() != 0) objectDpName += richtung + "_";
    if(gruppe.length() != 0) objectDpName += gruppe + "_";

    objectDpName += object;

    objectDpName.replace(",", "");
    objectDpName.replace(".", "");

    return objectDpName;
  }

//--------------------------------------------------------------------------------
//@private members
//--------------------------------------------------------------------------------

//--------------------------------------------------------------------------------
//@private data
//--------------------------------------------------------------------------------
  private shared_ptr<AKS> _objectAKS;
  private shared_ptr<AKS> _modeServeAKS;
  private shared_ptr<AKS> _modeControlAKS;
  private shared_ptr<AKS> _spezAKS;

  private string _objectDpName;
};
