// $License: NOLICENSE
//--------------------------------------------------------------------------------
/**
  @file $relPath
  @copyright $copyright
  @author mmarchhart
*/

//--------------------------------------------------------------------------------
// Libraries used (#uses)
#uses "classes/bsa/objectFactory/BsaObjectFactory"
#uses "classes/bsa/testObject/BsaTestObjectDefinitions"
#uses "classes/bsa/testObject/factory/BsaTestObjectFactoryInput"

//--------------------------------------------------------------------------------
// Variables and Constants

//--------------------------------------------------------------------------------
/**
*/
class BsaTestObjectFactory : BsaObjectFactory
{
//--------------------------------------------------------------------------------
//@public members
//--------------------------------------------------------------------------------

  //------------------------------------------------------------------------------
  /** The Default Constructor.
  */
  public BsaTestObjectFactory()
  {
  }

  /**
    Creates a datapoint of type VMS_Camera and sets all requried information for a VmsCamera object (like alert handlers)
    Throws an exception in case the datapoint couldn't be created
    @param cameraDpName
  */
  public static void CreateTestObjectDP(shared_ptr<BsaTestObjectFactoryInput> factoryInput)
  {
    BsaObjectFactory::CreateBsaObjectDP(factoryInput, BSA_TESTOBJECT_DPT_NAME);

    SetTestObjectConfig(factoryInput);
    SetTestObjectAlertHdl(factoryInput.GetObjectDpName());
  }

//--------------------------------------------------------------------------------
//@protected members
//--------------------------------------------------------------------------------

//--------------------------------------------------------------------------------
//@private members
//--------------------------------------------------------------------------------
  private static void SetTestObjectAlertHdl(const string &objectDpName)
  {
    //mma: this are for the meanwhile only some dummy alerts for testing/development purposes
    string alert1Dpe = objectDpName + ".alerts.alert_1";
    string alert2Dpe = objectDpName + ".alerts.alert_2";

    DeleteAlertHdl(alert1Dpe);
    DeleteAlertHdl(alert2Dpe);

    SetBoolAlert(alert1Dpe, "Alarm auf TestObject", "BSA_Alert");
    SetBoolAlert(alert2Dpe, "Störung auf TestObject", "BSA_Fault");
  }

  private static void SetTestObjectConfig(shared_ptr<BsaTestObjectFactoryInput> factoryInput)
  {
    string objectDpName = factoryInput.GetObjectDpName();

    dpSetAlias(objectDpName + ".alerts.alert_1", factoryInput.GetAlarm1AKSasString());
    dpSetAlias(objectDpName + ".alerts.alert_2", factoryInput.GetAlarm2AKSasString());
  }

};
