// $License: NOLICENSE
//--------------------------------------------------------------------------------
/**
  @file $relPath
  @copyright $copyright
  @author mmarchhart
*/

//--------------------------------------------------------------------------------
// Libraries used (#uses)
#uses "classes/sode/SodeMessageHandler"

//--------------------------------------------------------------------------------
// Variables and Constants

//--------------------------------------------------------------------------------
/**
*/
class SodeMessageHandlerFactory
{
//--------------------------------------------------------------------------------
//@public members
//--------------------------------------------------------------------------------
  public static shared_ptr<SodeMessageHandlerFactory> GetInstance()
  {
    if(_instance == nullptr)
    {
      _instance = new SodeMessageHandlerFactory();
    }

    return _instance;
  }

  public static shared_ptr<SodeMessageHandler> CreateMessageHandler(const string &host, int port)
  {
    shared_ptr<SodeMessageHandler> handler = new SodeMessageHandler(host, port);

    shared_ptr<TcpConnection> connection = handler.GetConnection();

    connection.RegisterReceiver(handler);

    shared_ptr<SodeMessageHandlerFactory> factory = GetInstance();
    factory.AddMessageHandler(handler);

    return handler;
  }

  public static void DisposeMessageHandler(shared_ptr<SodeMessageHandler> handler)
  {
    handler.Dispose();

    shared_ptr<SodeMessageHandlerFactory> factory = GetInstance();
    factory.RemoveMessageHandler(handler);
  }

//--------------------------------------------------------------------------------
//@protected members
//--------------------------------------------------------------------------------

//--------------------------------------------------------------------------------
//@private members
//--------------------------------------------------------------------------------
  //------------------------------------------------------------------------------
  /** The Default Constructor.
  */
  private SodeMessageHandlerFactory()
  {
    _stop = false;
    startThread(this, "Run");
  }

  ~SodeMessageHandlerFactory()
  {
    DebugTN(__FUNCTION__ + " stopping observation");
    _stop = true;
  }

  private void Run()
  {
    while(!_stop)
    {
      delay(1);

      synchronized(_handlerListMutex)
      {
        for(int i=0 ; i < _messageHandlers.count() ; ++i)
        {
          shared_ptr<SodeMessageHandler> handler = _messageHandlers.at(i);
          handler.CheckConnection();
        }
      }
    }

    DebugTN(__FUNCTION__ + " observation thread stopped");
  }

  private void AddMessageHandler(shared_ptr<SodeMessageHandler> handler)
  {
    synchronized(_handlerListMutex)
    {
      _messageHandlers.append(handler);
    }
    DebugTN(__FUNCTION__ + "# Instances: " + _messageHandlers.count());
  }

  private void RemoveMessageHandler(shared_ptr<SodeMessageHandler> handler)
  {
    int index = _messageHandlers.indexOf(handler);

    if(index != -1)
    {
      synchronized(_handlerListMutex)
      {
        _messageHandlers.removeAt(index);
      }
    }

    DebugTN(__FUNCTION__ + "# Instances: " + _messageHandlers.count());
  }

//--------------------------------------------------------------------------------
//@private data
//--------------------------------------------------------------------------------
  private bool _stop = false;
  private vector< shared_ptr<SodeMessageHandler> > _messageHandlers;
  private bool _handlerListMutex;

  private static shared_ptr<SodeMessageHandlerFactory> _instance;
};
