// $License: NOLICENSE
//--------------------------------------------------------------------------------
/**
  @file $relPath
  @copyright $copyright
  @author mmarchhart
*/

//--------------------------------------------------------------------------------
// Libraries used (#uses)
#uses "CtrlXml"

//--------------------------------------------------------------------------------
// Variables and Constants

struct CommandParameter
{
  CommandParameter(const string &name, const string &value)
  {
    Name = name;
    Value = value;
  }

  string Name;
  string Value;
};

//--------------------------------------------------------------------------------
/**
  This is supposed to be a abstract class for SODE commands that implements the parsing of commands to the xml format.
  When implementing a specific command override the GetCommandProperties() to return a list of additional command specific properties that should be included by the xml parser
*/
class BaseCommand
{
//--------------------------------------------------------------------------------
//@public members
//--------------------------------------------------------------------------------
  public BaseCommand(const string &name)
  {
    _name = name;
  }

  /**
    Serializes the command to an xml string
    @return
  */
  public string SerializeToXml()
  {
    int docId;

    docId = xmlNewDocument();

    int rootId = xmlAppendChild(docId, -1, XML_ELEMENT_NODE, "sode");
    int cmdNodeId = xmlAppendChild(docId, rootId, XML_ELEMENT_NODE, "command");

    xmlSetElementAttribute(docId, cmdNodeId, "name", _name);

    vector<CommandParameter> cmdParams = GetCommandParameters();  //mma: get now all additional parameters for the specific command

    for(int i=0 ; i < cmdParams.count() ; ++i)
    {
      CommandParameter param = cmdParams.at(i);

      int propNodeId = xmlAppendChild(docId, cmdNodeId, XML_ELEMENT_NODE, param.Name);
      int propValueId = xmlAppendChild(docId, propNodeId, XML_TEXT_NODE, param.Value);
    }

    string xmlDoc = xmlDocumentToString(docId);

    xmlCloseDocument(docId);

    return xmlDoc;
  }

  public string GetName()
  {
    return _name;
  }
//--------------------------------------------------------------------------------
//@protected members
//--------------------------------------------------------------------------------
  protected vector<CommandParameter> GetCommandParameters()
  {
    vector<CommandParameter> cmdParam;

    return cmdParam;
  }
//--------------------------------------------------------------------------------
//@private members
//--------------------------------------------------------------------------------

//--------------------------------------------------------------------------------
//@private data
//--------------------------------------------------------------------------------
  private string _name;
};
