// $License: NOLICENSE
//--------------------------------------------------------------------------------
/**
  @file $relPath
  @copyright $copyright
  @author mmarchhart
*/

//--------------------------------------------------------------------------------
// Libraries used (#uses)
#uses "classes/bsa/objectFactory/BsaObjectFactory"
#uses "classes/vms/cameraObject/VmsCameraObjectDefinitions"
#uses "classes/vms/cameraObject/factory/VmsCameraFactoryInput"

//--------------------------------------------------------------------------------
// Variables and Constants

//--------------------------------------------------------------------------------
/**
*/
class VmsCameraFactory : BsaObjectFactory
{
//--------------------------------------------------------------------------------
//@public members
//--------------------------------------------------------------------------------

  //------------------------------------------------------------------------------
  /** The Default Constructor.
  */
  public VmsCameraFactory()
  {
  }

  /**
    Creates a datapoint of type VMS_Camera and sets all requried information for a VmsCamera object (like alert handlers)
    Throws an exception in case the datapoint couldn't be created
    @param cameraDpName
  */
  public static void CreateVmsCameraDP(shared_ptr<VmsCameraFactoryInput> factoryInput)
  {
    BsaObjectFactory::CreateBsaObjectDP(factoryInput, VMS_CAMERA_DPT_NAME);

    SetCameraConfig(factoryInput);
    SetCameraAlertHdl(factoryInput);
  }

//--------------------------------------------------------------------------------
//@protected members
//--------------------------------------------------------------------------------

//--------------------------------------------------------------------------------
//@private members
//--------------------------------------------------------------------------------
  private static void SetCameraAlertHdl(shared_ptr<VmsCameraFactoryInput> factoryInput)
  {
    shared_ptr<AKS> aks = factoryInput.GetObjectAKS();

    string cameraDpName = factoryInput.GetObjectDpName();
    string alert1Dpe = cameraDpName + ".alerts.alert_1";

    DeleteAlertHdl(alert1Dpe);

    dpSetTimedWait(0, alert1Dpe + ":_alert_hdl.._type", DPCONFIG_ALERT_NONBINARYSIGNAL);  //mma: enable discreet alert handling

    dyn_string details;
    dyn_mixed values;

    string goodText = "Kamera " + aks.GetErfassungsbereich() + " " +
                                   aks.GetStreckenabschnitt() + " " +
                                   aks.GetAbschnitt() + " " +
                                   aks.GetRichtung() + " " +
                                   aks.GetObjekt() + " keine Störung";

    string alertText = "Kamera " + aks.GetErfassungsbereich() + " " +
                                   aks.GetStreckenabschnitt() + " " +
                                   aks.GetAbschnitt() + " " +
                                   aks.GetRichtung() + " " +
                                   aks.GetObjekt() + " Störung";

    AddDiscreteAlertRange(details, values, alert1Dpe, 1, 0, goodText, "");  //mma: add the good range
    AddDiscreteAlertRange(details, values, alert1Dpe, 2, 1, alertText, "BSA_Fault");

    dpSetTimedWait(0, details, values);

    dpSetTimedWait(0, alert1Dpe + ":_alert_hdl.._active", true);  //mma: activate now the alert hdl
  }

  private static void SetCameraConfig(shared_ptr<VmsCameraFactoryInput> factoryInput)
  {
    string cameraDpName = factoryInput.GetObjectDpName();

    dpSet(cameraDpName + ".config.url", factoryInput.GetUrl(),
          cameraDpName + ".config.monitorText", factoryInput.GetMonitorText(),
          cameraDpName + ".config.host", factoryInput.GetHost());

    dpSetAlias(cameraDpName + ".alerts.alert_1", factoryInput.GetAlarm1AKSasString());
  }

};
