// $License: NOLICENSE
//--------------------------------------------------------------------------------
/**
  @file $relPath
  @copyright $copyright
  @author mmarchhart
*/

//--------------------------------------------------------------------------------
// Libraries used (#uses)

//--------------------------------------------------------------------------------
// Variables and Constants

enum SodeTextPosition
{
  TOP,
  CENTER,
  BOTTOM,
  TOP_LEFT,
  TOP_RIGTH,
  BOTTOM_LEFT,
  BOTTOM_RIGHT
};

//--------------------------------------------------------------------------------
/**
*/
struct SodeTextConfig
{
//--------------------------------------------------------------------------------
//@public members
//--------------------------------------------------------------------------------

  //------------------------------------------------------------------------------
  /** The Default Constructor.
  */
  public SodeTextConfig(SodeTextPosition pos = SodeTextPosition::TOP, int size = 72, string color = "255,255,255")
  {
    Position = pos;
    Size = size;
    Color = color;
  }

  public string ToJson()
  {
    return jsonEncode(this);
  }

  public static shared_ptr<SodeTextConfig> FromJson(const string &json)
  {
    shared_ptr<SodeTextConfig> textConfig = nullptr;

    if(json != "")
    {
      try
      {
        mapping jsonData = jsonDecode(json);

        textConfig = new SodeTextConfig(jsonData.value("Position"),
                                    jsonData.value("Size"),
                                    jsonData.value("Color"));
      }
      catch
      {
        throw(makeError("", PRIO_WARNING, ERR_IMPL, 0, "Couldn't decode given json to an SodeTextConfig object: " + json));
      }
    }
    else
    {
      textConfig = new SodeTextConfig();
    }

    return textConfig;
  }

  public string PositionAsString()
  {
    return PositionToString(Position);
  }

  public static string PositionToString(SodeTextPosition textPos)
  {
    string position;

    switch(textPos)
    {
      case SodeTextPosition::TOP :
        position = "top";
        break;

      case SodeTextPosition::CENTER :
        position = "center";
        break;

      case SodeTextPosition::BOTTOM :
        position = "bottom";
        break;

      case SodeTextPosition::TOP_LEFT :
        position = "top-left";
        break;

      case SodeTextPosition::TOP_RIGTH :
        position = "top-right";
        break;

      case SodeTextPosition::BOTTOM_LEFT :
        position = "bottom-left";
        break;

      case SodeTextPosition::BOTTOM_RIGHT :
        position = "bottom-right";
        break;
    }

    return position;
  }

  public static SodeTextPosition StringToPosition(const string &position)
  {
    SodeTextPosition textPos = SodeTextPosition::TOP;

    switch(position)
    {
      case "top" :
        textPos = SodeTextPosition::TOP;
        break;

      case "center" :
        textPos = SodeTextPosition::CENTER;
        break;

      case "bottom" :
        textPos = SodeTextPosition::BOTTOM;
        break;

      case "top-left" :
        textPos = SodeTextPosition::TOP_LEFT;
        break;

      case "top-right" :
        textPos = SodeTextPosition::TOP_RIGTH;
        break;

      case "bottom-left" :
        textPos = SodeTextPosition::BOTTOM_LEFT;
        break;

      case "bottom-right" :
        textPos = SodeTextPosition::BOTTOM_RIGHT;
        break;
    }

    return textPos;
  }

//--------------------------------------------------------------------------------
//@protected members
//--------------------------------------------------------------------------------

//--------------------------------------------------------------------------------
//@public data
//--------------------------------------------------------------------------------
  public SodeTextPosition Position;
  public int Size;
  public string Color;

};
