// $License: NOLICENSE
//--------------------------------------------------------------------------------
/**
  @file $relPath
  @copyright $copyright
  @author mmarchhart
*/

//--------------------------------------------------------------------------------
// Libraries used (#uses)
#uses "classes/bsa/BsaObject"
#uses "classes/vms/sodeObject/SodeDisplayInfo"
#uses "classes/vms/sodeObject/SodeTextConfig"

//--------------------------------------------------------------------------------
// Variables and Constants

//--------------------------------------------------------------------------------
/**
*/
class VmsSodeHMIObject : BsaObject
{
//--------------------------------------------------------------------------------
//@public members
//--------------------------------------------------------------------------------

  //------------------------------------------------------------------------------
  /** The Default Constructor.
  */
  public VmsSodeHMIObject(const string &sodeObjectDpName) :
      BsaObject(sodeObjectDpName)
  {
  }

  public string GetMonitorNumber()
  {
    string monitorNumber;
    shared_ptr<AKS> objectAKS = GetObjectAKS();

    monitorNumber = objectAKS.GetObjekt();

    monitorNumber.replace("x", "");

    return monitorNumber;
  }

  public void SetActive(bool active)
  {
    dpSet(GetObjectDpName() + ".set.state", active);
  }

  public bool GetActive()
  {
    bool active;

    dpGet(GetObjectDpName() + ".current.state", active);

    return active;
  }

  public void SetDisplay(shared_ptr<SodeDisplayInfo> displayInfo)
  {
    dpSet(GetObjectDpName() + ".set.display", displayInfo.ToJson());
  }

  public shared_ptr<SodeDisplayInfo> GetDisplay()
  {
    string displayJson;
    dpGet(GetObjectDpName() + ".current.display", displayJson);

    return SodeDisplayInfo::FromJson(displayJson);
  }

  public void SetTextConfig(shared_ptr<SodeTextConfig> textConfig)
  {
    dpSet(GetObjectDpName() + ".set.textConfig", textConfig.ToJson());
  }

  public shared_ptr<SodeTextConfig> GetTextConfig()
  {
    string textConfigJson;
    dpGet(GetObjectDpName() + ".current.textConfig", textConfigJson);

    return SodeTextConfig::FromJson(textConfigJson);
  }

  /**
    Registers a listener to changes on the state of the SODE
     The reference panel must implement public void SodeStateChangedCB(bool active) to be a valid listener
    @param listener
    @return
  */
  public bool RegisterStateChangeListener(shape listener)
  {
    if(listener != nullptr)
    {
      synchronized(_stateChangeListenerMutex)
      {
        _stateChangeListener.append(listener);

        if(_stateChangeListener.count() == 1)  //mma: if it is the first listener who registers, establish the dpConnect
        {
          dpConnect(this, StateChangedCB, GetObjectDpName() + ".current.state");
        }
        else   //if the connect is already established, send the initial value to the new listener
        {
          listener.SodeStateChangedCB(_active);
        }
      }
    }
  }

  /**
    Registers a listener to changes on the display content (camara, text) of the SODE
     The reference panel must implement public void SodeDisplayChangedCB(shared_ptr<SodeDisplayInfo> displayInfo) to be a valid listener
    @param listener
    @return
  */
  public bool RegisterDisplayChangeListener(shape listener)
  {
    if(listener != nullptr)
    {
      synchronized(_displayChangeListenerMutex)
      {
        _displayChangeListener.append(listener);

        if(_displayChangeListener.count() == 1)  //mma: if it is the first listener who registers, establish the dpConnect
        {
          dpConnect(this, DisplayChangedCB, GetObjectDpName() + ".current.display");
        }
        else   //if the connect is already established, send the initial value to the new listener
        {
          listener.SodeDisplayChangedCB(_currentDisplay);
        }
      }
    }
  }
//--------------------------------------------------------------------------------
//@protected members
//--------------------------------------------------------------------------------

//--------------------------------------------------------------------------------
//@private members
//--------------------------------------------------------------------------------
  private void StateChangedCB(string dp1, bool active)
  {
    synchronized(_stateChangeListenerMutex)
    {
      _active = active;

      for(int i=0 ; i < _stateChangeListener.count() ; ++i)  //mma: inform all registered listeners about the changed state
      {
        shape listener = _stateChangeListener.at(i);
        listener.SodeStateChangedCB(_active);
      }
    }
  }

  private void DisplayChangedCB(string dp1, string displayInfoJson)
  {
    shared_ptr<SodeDisplayInfo> displayInfo = SodeDisplayInfo::FromJson(displayInfoJson);

    synchronized(_displayChangeListenerMutex)
    {
      assignPtr(_currentDisplay, displayInfo);

      for(int i=0 ; i < _displayChangeListener.count() ; ++i)  //mma: inform all registered listeners about the changed display info
      {
        shape listener = _displayChangeListener.at(i);
        listener.SodeDisplayChangedCB(_currentDisplay);
      }
    }
  }

//--------------------------------------------------------------------------------
//@private data
//--------------------------------------------------------------------------------
  private vector<shape> _stateChangeListener;
  private bool _stateChangeListenerMutex;
  private bool _active;

  private vector<shape> _displayChangeListener;
  private bool _displayChangeListenerMutex;
  private shared_ptr<SodeDisplayInfo> _currentDisplay;
};
