// $License: NOLICENSE
//--------------------------------------------------------------------------------
/**
  @file $relPath
  @copyright $copyright
  @author mmarchhart
*/

//--------------------------------------------------------------------------------
// Libraries used (#uses)
#uses "classes/bsa/objectFactory/BsaObjectFactory"
#uses "classes/vms/sodeObject/VmsSodeObjectDefinitions"
#uses "classes/vms/sodeObject/factory/VmsSodeFactoryInput"

//--------------------------------------------------------------------------------
// Variables and Constants

//--------------------------------------------------------------------------------
/**
*/
class VmsSodeFactory : BsaObjectFactory
{
//--------------------------------------------------------------------------------
//@public members
//--------------------------------------------------------------------------------

  //------------------------------------------------------------------------------
  /** The Default Constructor.
  */
  public VmsSodeFactory()
  {
  }

  /**
    Creates a datapoint of type VMS_Sode and sets all requried information for a VmsSode object (like alert handlers)
    Throws an exception in case the datapoint couldn't be created
    @param cameraDpName
  */
  public static void CreateVmsSodeDP(shared_ptr<VmsSodeFactoryInput> factoryInput)
  {
    BsaObjectFactory::CreateBsaObjectDP(factoryInput, VMS_SODE_DPT_NAME);

    SetSodeConfig(factoryInput);
    SetSodeAlertHdl(factoryInput);
  }

//--------------------------------------------------------------------------------
//@protected members
//--------------------------------------------------------------------------------

//--------------------------------------------------------------------------------
//@private members
//--------------------------------------------------------------------------------
  private static void SetSodeAlertHdl(shared_ptr<VmsSodeFactoryInput> factoryInput)
  {
    shared_ptr<AKS> aks = factoryInput.GetObjectAKS();

    string sodeDpName = factoryInput.GetObjectDpName();
    string alert1Dpe = sodeDpName + ".alerts.alert_1";

    DeleteAlertHdl(alert1Dpe);

    dpSetTimedWait(0, alert1Dpe + ":_alert_hdl.._type", DPCONFIG_ALERT_NONBINARYSIGNAL);  //mma: enable discreet alert handling

    dyn_string details;
    dyn_mixed values;

    string goodText = "Softwaredecoder " + aks.GetErfassungsbereich() + " " +
                                            aks.GetStreckenabschnitt() + " " +
                                            aks.GetAbschnitt() + " " +
                                            aks.GetObjekt() + " keine Störung";

    string alertText = "Softwaredecoder " + aks.GetErfassungsbereich() + " " +
                                            aks.GetStreckenabschnitt() + " " +
                                            aks.GetAbschnitt() + " " +
                                            aks.GetObjekt() + " Störung";

    AddDiscreteAlertRange(details, values, alert1Dpe, 1, 0, goodText, "");  //mma: add the good range
    AddDiscreteAlertRange(details, values, alert1Dpe, 2, 1, alertText, "BSA_Fault");

    dpSetTimedWait(0, details, values);

    dpSetTimedWait(0, alert1Dpe + ":_alert_hdl.._active", true);  //mma: activate now the alert hdl
  }

  private static void SetSodeConfig(shared_ptr<VmsSodeFactoryInput> factoryInput)
  {
    string sodeDpName = factoryInput.GetObjectDpName();

    dpSet(sodeDpName + ".config.host", factoryInput.GetHost(),
          sodeDpName + ".config.port", factoryInput.GetPort());

    dpSetAlias(sodeDpName + ".alerts.alert_1", factoryInput.GetAlarm1AKSasString());
  }

};
