// $License: NOLICENSE
//--------------------------------------------------------------------------------
/**
  @file $relPath
  @copyright $copyright
  @author mmarchhart
*/

//--------------------------------------------------------------------------------
// Libraries used (#uses)
#uses "classes/vms/sodeObject/sodeInterfaceHandler/SodeObjectInterface"
#uses "classes/vms/sodeObject/VmsSodeObjectDefinitions"

//--------------------------------------------------------------------------------
// Variables and Constants

//--------------------------------------------------------------------------------
/**
*/
class SodeInterfaceHandler
{
//--------------------------------------------------------------------------------
//@public members
//--------------------------------------------------------------------------------

  //------------------------------------------------------------------------------
  /** The Default Constructor.
  */
  public SodeInterfaceHandler()
  {
    _running = false;
    _stop = false;

    dyn_string sodeDPs = dpNames("*", VMS_SODE_DPT_NAME);
    DebugTN("Initialize sode instances: ", sodeDPs);

    for(int i=0 ; i < sodeDPs.count() ; ++i)  //mma: instanciate here for each defined sode object a SodeObjectInterface
    {
      _sodeObjectInterfaces.append(new SodeObjectInterface(sodeDPs.at(i)));
    }
  }

  /**
    Starts the main loop of the sode interface handler
  */
  public void Run()
  {
    if(_running) return;

    _running = true;
    _stop = false;

    DebugTN(__FUNCTION__ + " starting");

    for(int i=0 ; i < _sodeObjectInterfaces.count() ; ++i)  //mma: first start each sode interface
    {
      shared_ptr<SodeObjectInterface> sodeInterface = _sodeObjectInterfaces.at(i);
      sodeInterface.Start();
    }

    DebugTN(__FUNCTION__ + " running");

    while(!_stop)
    {
      for(int i=0 ; i < _sodeObjectInterfaces.count() ; ++i)  //mma: in the main loop, call periodically the ProcessInterface() method
      {
        shared_ptr<SodeObjectInterface> sodeInterface = _sodeObjectInterfaces.at(i);
        sodeInterface.ProcessInterface();
      }

      delay(5,0);
    }

    DebugTN(__FUNCTION__ + " stopping");

    _running = false;
  }

  /**
    Stopps the main loop of the sodo interface handler
  */
  public void Stop()
  {
    DebugTN(__FUNCTION__ + " got stop request");

    _stop = true;

    for(int i=0 ; i < _sodeObjectInterfaces.count() ; ++i)  //mma: stop each sode interface
    {
      shared_ptr<SodeObjectInterface> sodeInterface = _sodeObjectInterfaces.at(i);
      sodeInterface.Stop();
    }

    DebugTN(__FUNCTION__ + " clearing list");

    _sodeObjectInterfaces.clear();

    DebugTN(__FUNCTION__ + " stop done");
  }
//--------------------------------------------------------------------------------
//@protected members
//--------------------------------------------------------------------------------

//--------------------------------------------------------------------------------
//@private members
//--------------------------------------------------------------------------------

//--------------------------------------------------------------------------------
//@private data
//--------------------------------------------------------------------------------
  private vector< shared_ptr<SodeObjectInterface> > _sodeObjectInterfaces;
  private bool _running;
  private bool _stop;

};
